# -*- coding: UTF-8 -*-
"""
Name: renamer.py
Porpose: batch and single file renamer
Compatibility: Python3, wxPython Phoenix
Author: Gianluca Pernigotto <jeanlucperni@gmail.com>
Copyleft - 2024 Gianluca Pernigotto <jeanlucperni@gmail.com>
license: GPL3
Rev: July.17.2022
Code checker: flake8, pylint

This file is part of Videomass.

   Videomass is free software: you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation, either version 3 of the License, or
   (at your option) any later version.

   Videomass is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with Videomass.  If not, see <http://www.gnu.org/licenses/>.
"""
import re
import wx


class Renamer(wx.Dialog):
    """
    Simple file renaming dialog.
    It can be used both for renaming individual
    items and groups of items (batch mode). It has
    a dynamic interface for both modes.

    Usage example:
            with Renamer(self,
                         nameprop='New Name',
                         caption='My awesome title',
                         message='My message here',
                         mode=0,
                         ) as dlg:
                if dlg.ShowModal() == wx.ID_OK:
                    newname = dlg.getvalue()

    See __init__ docstring for parameter details.

    """

    def __init__(self,
                 parent,
                 nameprop=str('New name'),
                 caption=str('Caption'),
                 message=str('Message'),
                 mode=int(0),
                 ):
        """
        Parameters:

        parent (wx.Window) – Parent window. Must not be None.
        nameprop (string) – Proposed name, default value as text.
        caption (string) – title header on top of window.
        message (string) – the message you want to see.
        mode (int) – the "mode" parameter determines the type of renaming
                     See below for specification.

        mode=n (if n >= 1) rename multiple items as many as the given number.
        mode=n (if n == 0) rename an item only.
        Default is 0
        Note that mode=1 means to rename two filenames.

        """
        self.mode = mode
        msg = _("# will be replaced by ascending numbers starting with:")
        width = 472
        wx.Dialog.__init__(self, parent, -1, style=wx.DEFAULT_DIALOG_STYLE)
        sizer_base = wx.BoxSizer(wx.VERTICAL)
        sizer_base.Add((10, 10))
        labhead = wx.StaticText(self, wx.ID_ANY, message)
        sizer_base.Add(labhead, 0, wx.EXPAND | wx.ALL, 5)
        self.entry = wx.TextCtrl(self, wx.ID_ANY,
                                 nameprop,
                                 size=(width, -1)
                                 )
        sizer_base.Add(self.entry, 0, wx.EXPAND | wx.ALL, 5)
        boxsiz = wx.BoxSizer(wx.HORIZONTAL)
        sizer_base.Add(boxsiz, 0, wx.EXPAND)
        if self.mode >= 1:
            labnum = wx.StaticText(self, wx.ID_ANY, msg)
            boxsiz.Add(labnum, 0, wx.ALL | wx.ALIGN_CENTER, 5)
            self.prognum = wx.SpinCtrl(self, wx.ID_ANY,
                                       value="1",
                                       min=0, max=999999999,
                                       style=wx.SP_ARROW_KEYS,
                                       )
            boxsiz.Add(self.prognum, 0, wx.ALL | wx.ALIGN_CENTER, 5)
            width = labnum.GetSize()[0] + self.prognum.GetSize()[0]
            self.entry.SetSize(width, -1)

        # confirm buttons:
        gridexit = wx.BoxSizer(wx.HORIZONTAL)
        btn_close = wx.Button(self, wx.ID_CANCEL, "")
        gridexit.Add(btn_close, 0)
        self.btn_ok = wx.Button(self, wx.ID_OK, "")
        gridexit.Add(self.btn_ok, 0, wx.LEFT, 5)
        sizer_base.Add(gridexit, 0, wx.ALL | wx.ALIGN_RIGHT | wx.RIGHT, 5)

        self.SetTitle(caption)
        # self.SetMinSize((width, -1))
        self.SetSizer(sizer_base)
        sizer_base.Fit(self)
        self.Layout()
        # ----------------------Binding (EVT)--------------------------#

        self.Bind(wx.EVT_TEXT, self.on_entry_text, self.entry)
        self.Bind(wx.EVT_BUTTON, self.on_close, btn_close)
        self.Bind(wx.EVT_BUTTON, self.on_ok, self.btn_ok)
    # ------------------------------------------------------------------#

    def get_newname(self):
        """
        Set newname attribute.
        Return `list(newname)` if mode >= 1,
        Return `str(newname)` if mode == 0,
        Return `None` if raise ValueError.
        """
        string = self.entry.GetValue()
        if self.mode >= 1:
            startfrom = self.prognum.GetValue()
            try:
                lasthashchar = string.rindex('#')
            except ValueError:
                return None

            newname = [string for x in range(self.mode)]
            for num, name in enumerate(newname):
                temp = list(name)
                temp[lasthashchar] = str(startfrom + num)
                newname[num] = re.sub('#', '0', ''.join(temp))
        else:
            newname = string

        return newname
    # ------------------------------------------------------------------#

    def rename_file(self):
        """
        Checks for consistency of the entered string
        in one file renaming.
        """
        if self.entry.GetValue() == '' or self.entry.GetValue().isspace():
            self.btn_ok.Disable()
            return

        self.btn_ok.Enable()
    # ------------------------------------------------------------------#

    def rename_batch(self):
        """
        Checks for consistency of the entered string
        in batch renaming.
        """
        string = self.entry.GetValue()
        occur = re.findall(r'((\#)\#*)', string)

        if '#' not in string or len(occur) > 1:
            self.btn_ok.Disable()
            return

        self.btn_ok.Enable()
    # ----------------------Event handler (callback)----------------------#

    def on_entry_text(self, event):
        """
        Updates datas given from rename_batch/rename_file,
        during TextCtrl event.
        """
        if self.mode >= 1:
            self.rename_batch()
        else:
            self.rename_file()
    # ------------------------------------------------------------------#

    def on_close(self, event):
        """
        Close this dialog without saving anything
        """
        event.Skip()
    # ------------------------------------------------------------------#

    def on_ok(self, event):
        """
        Accept and close this dialog.
        Don't use self.Destroy() here.
        """
        event.Skip()
    # ------------------------------------------------------------------#

    def getvalue(self):
        """
        This method return values via the getvalue() interface
        from the caller. See Renamer class docstring or the caller
        for more info and usage.
        """
        return self.get_newname()



# -*- coding: UTF-8 -*-
"""
Name: get_bmpfromSvg.py
Porpose: return bmp image from a scalable vector graphic format (svg)
Compatibility: Python3, wxPython Phoenix
Author: Gianluca Pernigotto <jeanlucperni@gmail.com>
Copyleft - 2024 Gianluca Pernigotto <jeanlucperni@gmail.com>
license: GPL3
Rev: Feb.13.2024
Code checker: flake8, pylint

This file is part of Videomass.

   Videomass is free software: you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation, either version 3 of the License, or
   (at your option) any later version.

   Videomass is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with Videomass.  If not, see <http://www.gnu.org/licenses/>.
"""
try:
    from wx.svg import SVGimage
except ModuleNotFoundError:
    pass


def get_bmp(imgfile, size):
    """
    Given a file and a size, converts to bmp

    """
    img = SVGimage.CreateFromFile(imgfile)
    bmp = img.ConvertToScaledBitmap(size)

    return bmp



# -*- coding: UTF-8 -*-
"""
Name: video_to_sequence.py
Porpose: A simple images extractor UI
Compatibility: Python3, wxPython Phoenix
Author: Gianluca Pernigotto <jeanlucperni@gmail.com>
Copyleft - 2024 Gianluca Pernigotto <jeanlucperni@gmail.com>
license: GPL3
Rev: Mar.08.2024
Code checker: flake8, pylint

This file is part of Videomass.

   Videomass is free software: you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation, either version 3 of the License, or
   (at your option) any later version.

   Videomass is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with Videomass.  If not, see <http://www.gnu.org/licenses/>.
"""
import os
import sys
import wx
import wx.lib.agw.hyperlink as hpl
import wx.lib.agw.floatspin as FS
from videomass.vdms_dialogs.widget_utils import NormalTransientPopup
from videomass.vdms_utils.get_bmpfromsvg import get_bmp
from videomass.vdms_dialogs.filter_scale import Scale
from videomass.vdms_io.checkup import check_files
from videomass.vdms_dialogs.epilogue import Formula
from videomass.vdms_utils.utils import trailing_name_with_prog_digit
from videomass.vdms_utils.utils import update_timeseq_duration


class VideoToSequence(wx.Panel):
    """
    A simple UI for extracting frames from videos,
    with a special automation for output files and
    abilities of customization.
    """
    VIOLET = '#D64E93'
    LGREEN = '#52ee7d'
    BLACK = '#1f1f1f'
    MSG_1 = _("\n1. Import one or more video files, then select one."
              "\n\n2. To select a slice of time use the Timeline editor "
              "(CTRL+T) by adjusting the\n\"End\" and the \"Start\" duration "
              "values."
              "\n\n3. Select an output format (jpg, png, bmp)."
              "\n\n4. Start the conversion."
              "\n\n\nThe images produced will be saved in a folder "
              "named 'Movie_to_Pictures'\nwith a progressive digit, "
              "in the path you specify.")
    # ----------------------------------------------------------------#

    def __init__(self, parent):
        """
        This is a panel impemented on MainFrame
        """
        self.parent = parent  # parent is the MainFrame
        get = wx.GetApp()  # get data from bootstrap
        self.appdata = get.appset
        icons = get.iconset
        self.opt = {"Scale": "scale=w=320:h=-1", "Setdar": "", "Setsar": ""}

        if 'wx.svg' in sys.modules:  # available only in wx version 4.1 to up
            bmpresize = get_bmp(icons['scale'], ((16, 16)))
            self.bmpreset = get_bmp(icons['clear'], ((16, 16)))
        else:
            bmpresize = wx.Bitmap(icons['scale'], wx.BITMAP_TYPE_ANY)
            self.bmpreset = wx.Bitmap(icons['clear'], wx.BITMAP_TYPE_ANY)

        wx.Panel.__init__(self, parent, -1, style=wx.BORDER_THEME)
        sizer = wx.BoxSizer(wx.VERTICAL)
        sizer.Add((20, 20))
        btn_help = wx.Button(self, wx.ID_ANY, _("Read me"), size=(-1, -1))
        btn_help.SetBackgroundColour(wx.Colour(VideoToSequence.LGREEN))
        btn_help.SetForegroundColour(wx.Colour(VideoToSequence.BLACK))
        sizer.Add(btn_help, 0, wx.ALL, 5)
        sizer.Add((20, 20))
        choices = [(_('Create thumbnails')),
                   (_('Create tiled mosaics')),
                   (_('Create animated GIF')),
                   ]
        self.rdbx_opt = wx.RadioBox(self, wx.ID_ANY, (_("Options")),
                                    choices=choices,
                                    majorDimension=1,
                                    style=wx.RA_SPECIFY_ROWS,
                                    )
        sizer.Add(self.rdbx_opt, 0, wx.ALL | wx.EXPAND, 5)
        self.rdbx_opt.SetSelection(0)
        boxctrl = wx.StaticBoxSizer(wx.StaticBox(self, wx.ID_ANY), wx.VERTICAL)
        sizer.Add(boxctrl, 0, wx.ALL | wx.EXPAND, 5)
        siz_format = wx.BoxSizer(wx.HORIZONTAL)
        boxctrl.Add(siz_format)
        siz_ctrl = wx.BoxSizer(wx.HORIZONTAL)
        boxctrl.Add(siz_ctrl)
        self.lbl_rate = wx.StaticText(self, wx.ID_ANY, label="FPS:")
        siz_ctrl.Add(self.lbl_rate, 0, wx.LEFT | wx.ALIGN_CENTER_VERTICAL, 5)
        self.spin_rate = FS.FloatSpin(self, wx.ID_ANY,
                                      min_val=0.1, max_val=30.0,
                                      increment=0.1, value=0.2,
                                      agwStyle=FS.FS_LEFT, size=(120, -1)
                                      )
        self.spin_rate.SetFormat("%f")
        self.spin_rate.SetDigits(1)
        siz_ctrl.Add(self.spin_rate, 0, wx.ALL, 5)
        self.btn_resize = wx.Button(self, wx.ID_ANY, _("Resize"),
                                    size=(-1, -1)
                                    )
        self.btn_resize.SetBitmap(bmpresize, wx.LEFT)
        siz_ctrl.Add(self.btn_resize, 0, wx.ALL | wx.ALIGN_CENTER_VERTICAL, 5)
        self.lbl_frmt = wx.StaticText(self, wx.ID_ANY,
                                      label=_("Output format:")
                                      )
        siz_ctrl.Add(self.lbl_frmt, 0, wx.LEFT | wx.ALIGN_CENTER_VERTICAL, 5)
        self.cmb_frmt = wx.ComboBox(self, wx.ID_ANY,
                                    choices=['jpeg', 'png', 'bmp', 'gif'],
                                    size=(160, -1), style=wx.CB_DROPDOWN
                                    | wx.CB_READONLY,
                                    )
        siz_ctrl.Add(self.cmb_frmt, 0, wx.ALL, 5)
        self.cmb_frmt.SetSelection(2)
        siz_tile = wx.FlexGridSizer(4, 4, 0, 0)
        boxctrl.Add(siz_tile, 0, wx.TOP | wx.BOTTOM, 10)
        self.lbl_rows = wx.StaticText(self, wx.ID_ANY, label=_("Rows:"))
        siz_tile.Add(self.lbl_rows, 0, wx.LEFT | wx.ALIGN_CENTER_VERTICAL, 5)
        self.lbl_rows.Disable()
        self.spin_rows = wx.SpinCtrl(self, wx.ID_ANY, "4", min=2,
                                     max=32, size=(-1, -1),
                                     style=wx.TE_PROCESS_ENTER
                                     )
        siz_tile.Add(self.spin_rows, 0, wx.ALL | wx.ALIGN_CENTER_VERTICAL, 5)
        self.spin_rows.Disable()
        self.lbl_cols = wx.StaticText(self, wx.ID_ANY,
                                      label=_("Columns:"))
        siz_tile.Add(self.lbl_cols, 0, wx.LEFT | wx.ALIGN_CENTER_VERTICAL, 5)
        self.lbl_cols.Disable()
        self.spin_cols = wx.SpinCtrl(self, wx.ID_ANY, "4", min=2,
                                     max=32, size=(-1, -1),
                                     style=wx.TE_PROCESS_ENTER
                                     )
        siz_tile.Add(self.spin_cols, 0, wx.ALL | wx.ALIGN_CENTER_VERTICAL, 5)
        self.spin_cols.Disable()
        self.lbl_pad = wx.StaticText(self, wx.ID_ANY, label="Padding:")
        siz_tile.Add(self.lbl_pad, 0, wx.LEFT | wx.ALIGN_CENTER_VERTICAL, 5)
        self.lbl_pad.Disable()
        self.spin_pad = wx.SpinCtrl(self, wx.ID_ANY, "2", min=0,
                                    max=32, size=(-1, -1),
                                    style=wx.TE_PROCESS_ENTER
                                    )
        siz_tile.Add(self.spin_pad, 0, wx.ALL | wx.ALIGN_CENTER_VERTICAL, 5)
        self.spin_pad.Disable()

        self.lbl_marg = wx.StaticText(self, wx.ID_ANY, label="Margin:")
        siz_tile.Add(self.lbl_marg, 0, wx.LEFT | wx.ALIGN_CENTER_VERTICAL, 5)
        self.lbl_marg.Disable()
        self.spin_marg = wx.SpinCtrl(self, wx.ID_ANY, "2", min=0,
                                     max=32, size=(-1, -1),
                                     style=wx.TE_PROCESS_ENTER
                                     )
        siz_tile.Add(self.spin_marg, 0, wx.ALL | wx.ALIGN_CENTER_VERTICAL, 5)
        self.spin_marg.Disable()
        siz_addparams = wx.BoxSizer(wx.HORIZONTAL)
        boxctrl.Add(siz_addparams, 0, wx.EXPAND, 0)
        self.ckbx_edit = wx.CheckBox(self, wx.ID_ANY, _('Edit'))
        siz_addparams.Add(self.ckbx_edit, 0, wx.ALL | wx.EXPAND, 5)
        self.txt_args = wx.TextCtrl(self, wx.ID_ANY, size=(700, -1),)
        siz_addparams.Add(self.txt_args, 1, wx.ALL | wx.EXPAND, 5)
        self.txt_args.Disable()
        sizer.Add((20, 20))
        fgs1 = wx.BoxSizer(wx.VERTICAL)
        sizer_link1 = wx.BoxSizer(wx.HORIZONTAL)
        fgs1.Add(sizer_link1)
        lbl_msg1 = wx.StaticText(self, wx.ID_ANY,
                                 label=_("For more information, "
                                         "visit the official FFmpeg "
                                         "documentation:")
                                 )
        link1 = hpl.HyperLinkCtrl(self, -1, "4.19 image2",
                                  URL="https://ffmpeg.org/ffmpeg-"
                                      "formats.html#image2-2"
                                  )
        link2 = hpl.HyperLinkCtrl(self, -1, ("3.3 FFmpeg FAQ"),
                                  URL="https://ffmpeg.org/faq.html#How-do-I-"
                                      "encode-movie-to-single-pictures_003f"
                                  )
        link3 = hpl.HyperLinkCtrl(self, -1, ("FFmpeg Wiki"),
                                  URL="https://trac.ffmpeg.org/wiki/Create%"
                                      "20a%20thumbnail%20image%20every"
                                      "%20X%20seconds%20of%20the%20video"
                                  )
        sizer_link1.Add(lbl_msg1, 0, wx.ALL | wx.EXPAND, 5)
        sizer_link1.Add(link1)
        sizer_link1.Add((20, 20))
        sizer_link1.Add(link2)
        sizer_link1.Add((20, 20))
        sizer_link1.Add(link3)
        sizer_link2 = wx.BoxSizer(wx.HORIZONTAL)
        fgs1.Add(sizer_link2)
        lbl_msg2 = wx.StaticText(self, wx.ID_ANY,
                                 label=_("Other unofficial resources:")
                                 )
        link4 = hpl.HyperLinkCtrl(self, -1, ("Help Tile filter"),
                                  URL="http://underpop.online.fr/f/ffmpeg"
                                      "/help/tile.htm.gz"
                                  )
        link5 = hpl.HyperLinkCtrl(self, -1, ("High quality gif"),
                                  URL="http://blog.pkh.me/p/21-high-quality-"
                                      "gif-with-ffmpeg.html"
                                  )
        sizer_link2.Add(lbl_msg2, 0, wx.ALL | wx.EXPAND, 5)
        sizer_link2.Add(link4)
        sizer_link2.Add((20, 20))
        sizer_link2.Add(link5)
        sizer.Add(fgs1, 0, wx.ALL | wx.EXPAND, 5)
        self.SetSizer(sizer)

        if self.appdata['ostype'] == 'Darwin':
            lbl_msg1.SetFont(wx.Font(11, wx.SWISS, wx.NORMAL, wx.NORMAL))
            lbl_msg2.SetFont(wx.Font(11, wx.SWISS, wx.NORMAL, wx.NORMAL))
        else:
            lbl_msg1.SetFont(wx.Font(8, wx.SWISS, wx.NORMAL, wx.NORMAL))
            lbl_msg2.SetFont(wx.Font(8, wx.SWISS, wx.NORMAL, wx.NORMAL))

        tip = (_('Set FPS control from 0.1 to 30.0 fps. The higher this '
                 'value, the more images will be extracted.'))
        self.spin_rate.SetToolTip(tip)
        tip = _('Spaces around the mosaic tiles. From 0 to 32 pixels')
        self.spin_pad.SetToolTip(tip)
        tip = _('Spaces around the mosaic borders. From 0 to 32 pixels')
        self.spin_marg.SetToolTip(tip)

        self.Bind(wx.EVT_BUTTON, self.on_help, btn_help)
        self.Bind(wx.EVT_CHECKBOX, self.on_edit, self.ckbx_edit)
        self.Bind(wx.EVT_RADIOBOX, self.on_options, self.rdbx_opt)
        self.Bind(wx.EVT_BUTTON, self.on_resizing, self.btn_resize)
    # --------------------------------------------------------------#

    def on_help(self, event):
        """
        event on button help
        """
        win = NormalTransientPopup(self,
                                   wx.SIMPLE_BORDER,
                                   VideoToSequence.MSG_1,
                                   VideoToSequence.LGREEN,
                                   VideoToSequence.BLACK)

        # Show the popup right below or above the button
        # depending on available screen space...
        btn = event.GetEventObject()
        pos = btn.ClientToScreen((0, 0))
        sz = btn.GetSize()
        win.Position(pos, (0, sz[1]))

        win.Popup()
    # --------------------------------------------------------------#

    def on_edit(self, event):
        """
        edit in place with additional paramemeters
        """
        if self.ckbx_edit.IsChecked():
            self.spin_rate.Disable()
            self.cmb_frmt.Disable()
            self.lbl_rate.Disable()
            self.lbl_frmt.Disable()
            self.txt_args.Enable()
            self.btn_resize.Disable()
            self.lbl_rows.Disable()
            self.lbl_cols.Disable()
            self.lbl_pad.Disable()
            self.lbl_marg.Disable()
            self.spin_rows.Disable()
            self.spin_cols.Disable()
            self.spin_pad.Disable()
            self.spin_marg.Disable()
            arg = self.update_arguments(self.cmb_frmt.GetValue())
            self.txt_args.write(arg[1])
        else:
            if self.rdbx_opt.GetSelection() == 0:
                self.spin_rate.Enable()
                self.lbl_rate.Enable()
            if self.rdbx_opt.GetSelection() in (0, 1):
                self.cmb_frmt.Enable()
                self.lbl_frmt.Enable()
                self.btn_resize.Enable()
                if self.rdbx_opt.GetSelection() == 1:
                    self.lbl_rows.Enable()
                    self.lbl_cols.Enable()
                    self.lbl_pad.Enable()
                    self.lbl_marg.Enable()
                    self.spin_rows.Enable()
                    self.spin_cols.Enable()
                    self.spin_pad.Enable()
                    self.spin_marg.Enable()
            elif self.rdbx_opt.GetSelection() == 2:
                self.btn_resize.Enable()

            self.txt_args.Clear()
            self.txt_args.Disable()
    # ------------------------------------------------------------------#

    def on_options(self, event):
        """
        Available user options
        """
        if self.rdbx_opt.GetSelection() == 0:
            self.cmb_frmt.SetSelection(2)
            self.txt_args.Clear()
            self.lbl_rows.Disable()
            self.lbl_cols.Disable()
            self.lbl_pad.Disable()
            self.lbl_marg.Disable()
            self.spin_rows.Disable()
            self.spin_cols.Disable()
            self.spin_pad.Disable()
            self.spin_marg.Disable()
            self.on_edit(self)

        elif self.rdbx_opt.GetSelection() == 1:
            self.cmb_frmt.SetSelection(1)
            self.txt_args.Clear()
            self.lbl_rate.Disable()
            self.spin_rate.Disable()
            self.lbl_rows.Enable()
            self.lbl_cols.Enable()
            self.lbl_pad.Enable()
            self.lbl_marg.Enable()
            self.spin_rows.Enable()
            self.spin_cols.Enable()
            self.spin_pad.Enable()
            self.spin_marg.Enable()
            self.on_edit(self)

        elif self.rdbx_opt.GetSelection() == 2:
            self.cmb_frmt.SetSelection(3)
            self.cmb_frmt.Disable()
            self.lbl_frmt.Disable()
            self.txt_args.Clear()
            self.lbl_rate.Disable()
            self.spin_rate.Disable()
            self.lbl_rows.Disable()
            self.lbl_cols.Disable()
            self.lbl_pad.Disable()
            self.lbl_marg.Disable()
            self.spin_rows.Disable()
            self.spin_cols.Disable()
            self.spin_pad.Disable()
            self.spin_marg.Disable()
            self.on_edit(self)
    # ------------------------------------------------------------------#

    def file_selection(self):
        """
        Gets the selected file on files list and returns an object
        of type list [str('selected file name'), int(index)].
        Returns None if no files are selected.

        """
        if len(self.parent.file_src) == 1:
            return (self.parent.file_src[0], 0)

        if not self.parent.filedropselected:
            wx.MessageBox(_("Have to select an item in the file list first"),
                          'Videomass', wx.ICON_INFORMATION, self)
            return None

        clicked = self.parent.filedropselected
        return (clicked, self.parent.file_src.index(clicked))
    # ------------------------------------------------------------------#

    def get_video_stream(self):
        """
        Given a frame or a video file, it returns a tuple of data
        containing information on the streams required by some video
        filters.

        """
        fget = self.file_selection()
        if not fget:
            return None

        index = self.parent.data_files[fget[1]]

        if 'video' in index.get('streams')[0]['codec_type']:
            width = int(index['streams'][0]['width'])
            height = int(index['streams'][0]['height'])
            filename = index['format']['filename']
            duration = index['format'].get('time', '00:00:00.000')
            return dict(zip(['width', 'height', 'filename', 'duration'],
                            [width, height, filename, duration]))

        wx.MessageBox(_('The file is not a frame or a video file'),
                      _('Videomass - Warning!'), wx.ICON_WARNING)
        return None
    # ------------------------------------------------------------------#

    def on_resizing(self, event):
        """
        Enable or disable scale, setdar and setsar filters
        """
        kwa = self.get_video_stream()
        if not kwa:
            return
        with Scale(self,
                   self.opt["Scale"],
                   self.opt["Setdar"],
                   self.opt["Setsar"],
                   self.bmpreset,
                   **kwa,
                   ) as sizing:

            if sizing.ShowModal() == wx.ID_OK:
                data = sizing.getvalue()
                if not [x for x in data.values() if x]:
                    self.btn_resize.SetBackgroundColour(wx.NullColour)
                    self.opt["Setdar"] = ""
                    self.opt["Setsar"] = ""
                    self.opt["Scale"] = "scale=w=320:h=-1"
                else:
                    self.btn_resize.SetBackgroundColour(
                        wx.Colour(VideoToSequence.VIOLET))
                    self.opt["Scale"] = data['scale']
                    self.opt['Setdar'] = data['setdar']
                    self.opt['Setsar'] = data['setsar']
    # ------------------------------------------------------------------#

    def update_arguments(self, fmt):
        """
        Given a image format return the corresponding
        ffmpeg argument
        """
        if self.rdbx_opt.GetSelection() == 1:
            rows = self.spin_rows.GetValue()
            cols = self.spin_cols.GetValue()
            pad = self.spin_pad.GetValue()
            marg = self.spin_marg.GetValue()
            scale = self.opt["Scale"]
            if self.opt["Setdar"]:
                scale = f'{scale},{self.opt["Setdar"]}'
            if self.opt["Setsar"]:
                scale = f'{scale},{self.opt["Setsar"]}'
            cmd = ('-skip_frame nokey', f'-vf "{scale},tile={rows}x{cols}:'
                   f'padding={pad}:margin={marg}:color=White" -an '
                   f'-fps_mode vfr')

        elif self.rdbx_opt.GetSelection() == 2:
            setf = ''
            if self.opt["Setdar"]:
                setf = f',{self.opt["Setdar"]}'
            if self.opt["Setsar"]:
                setf = f'{setf},{self.opt["Setsar"]}'
            flt = (f'fps=10,{self.opt["Scale"]}:flags=lanczos,split=2 '
                   f'[a][b]; [a] palettegen [pal]; [b] fifo [b]; [b] '
                   f'[pal] paletteuse{setf}')
            cmd = ('', f'-vf "{flt}" -loop 0')

        elif self.rdbx_opt.GetSelection() == 0:
            scale = self.opt["Scale"]
            if self.opt["Setdar"]:
                scale = f'{scale},{self.opt["Setdar"]}'
            if self.opt["Setsar"]:
                scale = f'{scale},{self.opt["Setsar"]}'

            arg = {'jpeg': ('',
                            f'-fps_mode cfr -r {self.spin_rate.GetValue()} '
                            f'-vf "{scale}" -pix_fmt yuvj420p'
                            ),
                   'png': ('',
                           f'-fps_mode cfr -r {self.spin_rate.GetValue()} '
                           f'-vf "{scale}" -pix_fmt rgb24'
                           ),
                   'bmp': ('',
                           f'-fps_mode cfr -r {self.spin_rate.GetValue()} '
                           f'-vf "{scale}" -pix_fmt bgr24'
                           ),
                   'gif': ('',
                           f'-fps_mode cfr -r {self.spin_rate.GetValue()} '
                           f'-vf "{scale}"'
                           )
                   }
            cmd = arg[fmt]

        return cmd
    # ------------------------------------------------------------------#

    def on_start(self):
        """
        Check before Builds FFmpeg command arguments
        """
        fsource = self.parent.file_src
        if len(fsource) == 1:
            clicked = fsource[0]

        elif not self.parent.filedropselected:
            wx.MessageBox(_("Have to select an item in the file list first"),
                          'Videomass', wx.ICON_INFORMATION, self)
            return
        else:
            clicked = self.parent.filedropselected

        getclk = fsource.index(clicked)
        typemedia = self.parent.fileDnDTarget.flCtrl.GetItemText(getclk, 3)

        if 'video' not in typemedia or 'sequence' in typemedia:
            wx.MessageBox(_("Invalid file: '{}'").format(clicked),
                          _('Videomass - Error!'), wx.ICON_ERROR, self)
            return

        checking = check_files((clicked,),
                               self.appdata['outputdir'],
                               self.appdata['outputdir_asinput'],
                               self.appdata['filesuffix'],
                               self.cmb_frmt.GetValue(),
                               self.parent.outputnames
                               )
        if not checking:  # User changing idea or not such files exist
            return

        self.build_args(clicked, checking[1])
    # ------------------------------------------------------------------#

    def build_args(self, filename, outfile):
        """
        Save as files image the selected video input. The saved
        images are named as file name + a progressive number + .jpg
        and placed in a folder with the same file name + a progressive
        number in the chosen output path.

        """
        destdir = os.path.dirname(outfile[0])  # specified dest
        outputdir = trailing_name_with_prog_digit(destdir, 'Movie_to_Pictures')

        if self.cmb_frmt.GetValue() != 'gif':
            namesplit = os.path.splitext(outfile[0])
            fileout = f"{namesplit[0]}_%d{namesplit[1]}"
        else:
            fileout = f"{outfile[0]}"

        outfilename = os.path.join(outputdir, os.path.basename(fileout))

        if self.txt_args.IsEnabled():
            arg = self.update_arguments(self.cmb_frmt.GetValue())
            preargs = arg[0]
            command = " ".join(f'{self.txt_args.GetValue()} -y '
                               f'"{outfilename}"'.split())
        else:
            arg = self.update_arguments(self.cmb_frmt.GetValue())
            preargs = arg[0]
            command = " ".join(f'{arg[1]} {self.txt_args.GetValue()} -y '
                               f'"{outfilename}"'.split())

        dur, ss, et = update_timeseq_duration(self.parent.time_seq,
                                              self.parent.duration
                                              )
        kwargs = {'logname': 'From Movie to Pictures.log',
                  'type': 'video_to_sequence', 'duration': dur,
                  'start-time': ss, 'end-time': et, 'filename': filename,
                  'outputdir': outputdir, 'args': command,
                  'pre-input-1': preargs,
                  'preset name': 'From Movies to Pictures',
                  }
        keyval = self.update_dict(filename, outputdir)
        ending = Formula(self, (700, 280),
                         self.parent.movetotrash,
                         self.parent.emptylist,
                         **keyval,
                         )
        if ending.ShowModal() == wx.ID_OK:
            (self.parent.movetotrash,
             self.parent.emptylist) = ending.getvalue()
        else:
            return

        try:
            os.makedirs(outputdir, mode=0o777)
        except (OSError, FileExistsError) as err:
            wx.MessageBox(f"{err}", _('Videomass - Error!'),
                          wx.ICON_ERROR, self)
            return

        self.parent.switch_to_processing(kwargs["type"],
                                         kwargs["logname"],
                                         datalist=kwargs
                                         )
        return
    # ------------------------------------------------------------------#

    def update_dict(self, filename, outputdir):
        """
        Update information before send to epilogue

        """
        if not self.parent.time_seq:
            sst, endt = _('Same as source'), _('Same as source')
        else:
            endt = self.parent.time_seq.split()[3]
            sst = self.parent.time_seq.split()[1]

        if self.txt_args.IsEnabled():
            args = _('Enabled')
            rate = 'Disabled'
            resize, rows, cols, pad, marg = '', '', '', '', ''
        else:
            args = _('Disabled')
            resize = (f'{self.opt["Scale"]} '
                      f'{self.opt["Setdar"]} '
                      f'{self.opt["Setsar"]}')
            if self.rdbx_opt.GetSelection() == 0:
                rate = self.spin_rate.GetValue()
                rows, cols, pad, marg = '', '', '', ''
            elif self.rdbx_opt.GetSelection() == 1:
                rate = ''
                rows = self.spin_rows.GetValue()
                cols = self.spin_cols.GetValue()
                pad = self.spin_pad.GetValue()
                marg = self.spin_marg.GetValue()
            elif self.rdbx_opt.GetSelection() == 2:
                rate, rows, cols, pad, marg = '', '', '', '', ''

        keys = (_("Selected File\nOutput Format\n"
                  "Destination\nRate (fps)\nResizing\n"
                  "Mosaic rows\nMosaic columns\nMosaic padding\n"
                  "Mosaic margin\nCustom Arguments\n"
                  "Start of segment\nDuration"
                  ))
        vals = (f"{filename}\n{self.cmb_frmt.GetValue()}\n{outputdir}"
                f"\n{rate}\n{resize}\n{rows}\n{cols}\n{pad}\n{marg}"
                f"\n{args}\n{sst}\n{endt}"
                )
        return {'key': keys, 'val': vals}



# -*- coding: UTF-8 -*-
"""
Name: shutdown.py
Porpose: Execute shutdown system using subprocess
Compatibility: Python3 (Unix, Windows)
Author: Gianluca Pernigotto <jeanlucperni@gmail.com>
Copyleft - 2024 Gianluca Pernigotto <jeanlucperni@gmail.com>
license: GPL3
Rev: May.11.2024
Code checker: flake8, pylint

This file is part of Videomass.

   Videomass is free software: you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation, either version 3 of the License, or
   (at your option) any later version.

   Videomass is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with Videomass.  If not, see <http://www.gnu.org/licenses/>.
"""
import platform
import subprocess
import wx
from videomass.vdms_utils.utils import Popen
from videomass.vdms_io.make_filelog import make_log_template


def logwrite(logfile, cmd):
    """
    write ffmpeg command log
    """
    with open(logfile, "a", encoding='utf-8') as log:
        log.write(f"{cmd}\n")


def logerror(logfile, output):
    """
    write ffmpeg volumedected errors
    """
    with open(logfile, "a", encoding='utf-8') as logerr:
        logerr.write(f"\nERRORS:\n{output}\n")


def shutdown_system(password=None):
    """
    Turn off the system using subprocess
    """
    get = wx.GetApp()
    appdata = get.appset
    logfile = make_log_template("Shutdown.log", appdata['logdir'], mode="w")

    if platform.system() == 'Windows':
        cmd = ["shutdown", "/s", "/t", "1"]
    else:
        if password:
            password = f"{password}\n"
            cmd = ["sudo", "-S", "/sbin/shutdown", "-h", "now"]
        else:
            cmd = ["/sbin/shutdown", "-h", "now"]
    try:
        with Popen(cmd,
                   stdin=subprocess.PIPE,
                   stdout=subprocess.PIPE,
                   stderr=subprocess.PIPE,
                   universal_newlines=True,
                   encoding='utf-8',
                   ) as proc:

            output = proc.communicate(password)[1]
            proc.wait()
            logwrite(logfile, output)
            return not output or output == "Password:"

    except (OSError, FileNotFoundError) as err:
        logerror(logfile, output)
        return err

    return None



# -*- coding: UTF-8 -*-
"""
Name: showlogs.py
Porpose: show logs data
Compatibility: Python3, wxPython Phoenix
Author: Gianluca Pernigotto <jeanlucperni@gmail.com>
Copyleft - 2024 Gianluca Pernigotto <jeanlucperni@gmail.com>
license: GPL3
Rev: July.17.2023
Code checker: flake8, pylint

This file is part of Videomass.

   Videomass is free software: you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation, either version 3 of the License, or
   (at your option) any later version.

   Videomass is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with Videomass.  If not, see <http://www.gnu.org/licenses/>.
"""
import os
import wx
from pubsub import pub


class ShowLogs(wx.Dialog):
    """
    View log data from files within the log directory

    """
    # list of logs files to include
    LOGNAMES = ('volumedetected.log',
                'AV Conversions.log',
                'Presets Manager.log',
                'ffplay.log',
                'Concatenate Media File.log',
                'From Movie to Pictures.log',
                'Still Image Maker.log',
                'generic_task.log',
                'YouTube Downloader.log',
                'Queue Processing.log',
                'Shutdown.log',
                )

    def __init__(self, parent, dirlog, OS):
        """
        Attributes defined here:
        self.dirlog > log location directory (depends from OS)
        self.logdata > dict object {KEY=file name.log: VAL=log data, ...}
        self.selected > None if item on listctrl is not selected
        file > log file name to view

        """
        self.dirlog = dirlog
        self.logdata = {}
        self.selected = None
        get = wx.GetApp()  # get data from bootstrap
        colorscheme = get.appset['colorscheme']
        vidicon = get.iconset['videomass']

        wx.Dialog.__init__(self, None,
                           style=wx.DEFAULT_DIALOG_STYLE
                           | wx.RESIZE_BORDER
                           | wx.DIALOG_NO_PARENT
                           )
        # ----------------------Layout----------------------#
        sizer_base = wx.BoxSizer(wx.VERTICAL)
        self.log_select = wx.ListCtrl(self,
                                      wx.ID_ANY,
                                      style=wx.LC_REPORT
                                      | wx.SUNKEN_BORDER
                                      | wx.LC_SINGLE_SEL,
                                      )
        self.log_select.SetMinSize((600, 200))
        self.log_select.InsertColumn(0, _('Log file list'), width=300)
        sizer_base.Add(self.log_select, 0, wx.ALL | wx.EXPAND, 5)
        lablog = wx.StaticText(self, label=_('Log messages'))
        sizer_base.Add(lablog, 0, wx.ALL, 5)
        self.textdata = wx.TextCtrl(self,
                                    wx.ID_ANY, "",
                                    style=wx.TE_MULTILINE
                                    | wx.TE_READONLY
                                    | wx.TE_RICH2,
                                    )
        self.textdata.SetMinSize((600, 200))
        self.textdata.SetBackgroundColour(colorscheme['BACKGRD'])
        self.textdata.SetDefaultStyle(wx.TextAttr(colorscheme['TXT3']))

        if OS == 'Darwin':
            self.textdata.SetFont(wx.Font(12, wx.MODERN, wx.NORMAL,
                                          wx.NORMAL))
        else:
            self.textdata.SetFont(wx.Font(9, wx.MODERN, wx.NORMAL, wx.NORMAL))

        sizer_base.Add(self.textdata, 1, wx.ALL | wx.EXPAND, 5)
        # ----- confirm buttons section
        grdBtn = wx.GridSizer(1, 2, 0, 0)
        grid_funcbtn = wx.BoxSizer(wx.HORIZONTAL)
        button_update = wx.Button(self, wx.ID_REFRESH,
                                  _("Refresh log messages"))
        grid_funcbtn.Add(button_update, 0)
        button_clear = wx.Button(self, wx.ID_CLEAR, _("Clear log messages"))
        grid_funcbtn.Add(button_clear, 0, wx.LEFT, 5)
        grdBtn.Add(grid_funcbtn, 0, wx.ALL, 5)
        grdexit = wx.BoxSizer(wx.HORIZONTAL)
        button_close = wx.Button(self, wx.ID_CLOSE, "")
        grdexit.Add(button_close, 0)
        grdBtn.Add(grdexit, flag=wx.ALL | wx.ALIGN_RIGHT | wx.RIGHT, border=5)
        sizer_base.Add(grdBtn, 0, wx.ALL | wx.EXPAND, 0)
        # set caption and min size
        self.SetTitle(_('Log messages'))
        self.SetMinSize((700, 500))
        icon = wx.Icon()
        icon.CopyFromBitmap(wx.Bitmap(vidicon, wx.BITMAP_TYPE_ANY))
        self.SetIcon(icon)
        # ------ set sizer
        self.SetSizer(sizer_base)
        self.Fit()
        self.Layout()

        # populate ListCtrl and set self.logdata dict
        self.on_update(self)

        # ----------------------Binding (EVT)----------------------#
        self.Bind(wx.EVT_LIST_ITEM_SELECTED, self.on_select, self.log_select)
        self.Bind(wx.EVT_LIST_ITEM_DESELECTED, self.on_deselect,
                  self.log_select)
        self.Bind(wx.EVT_BUTTON, self.on_update, button_update)
        self.Bind(wx.EVT_BUTTON, self.on_clear, button_clear)
        self.Bind(wx.EVT_BUTTON, self.on_close, button_close)
        self.Bind(wx.EVT_CLOSE, self.on_close)

    # ----------------------Event handler (callback)----------------------#

    # def on_flog_select(self, logfile):
    #     """
    #     Auto-Select a specified logfile name.
    #     FindItem method:
    #     <https://docs.wxpython.org/wx.ListCtrl.html#wx.ListCtrl.FindItem>
    #     """
    #     logname = os.path.basename(logfile)
    #     idx = self.log_select.FindItem(-1, logname)
    #
    #     if not idx  == -1:
    #         self.log_select.Focus(idx)  # make the line the current line
    #         self.log_select.Select(idx, on=1)  # default event selection
    # --------------------------------------------------------------------#

    def on_clear(self, event):
        """
        clear data logging from selected log file

        """
        if not self.selected:
            wx.MessageBox(_('Select a log file'),
                          'Videomass', wx.ICON_INFORMATION)
            return

        index = self.log_select.GetFocusedItem()
        name = self.log_select.GetItemText(index, 0)

        if wx.MessageBox(_('Are you sure you want to clear the selected '
                           'log file?'), _('Please confirm'), wx.ICON_QUESTION
                         | wx.CANCEL | wx.YES_NO, self) != wx.YES:
            return

        with open(os.path.join(self.dirlog, name),
                  'w', encoding='utf-8') as log:
            log.write('')

        self.on_update(self)
    # --------------------------------------------------------------------#

    def on_update(self, event):
        """
        update data with new incoming

        """
        sel = self.log_select.GetFocusedItem()
        selitem = sel if sel != -1 else 0

        self.logdata.clear()
        self.log_select.DeleteAllItems()
        index = 0
        for f in os.listdir(self.dirlog):
            if os.path.basename(f) in ShowLogs.LOGNAMES:  # append listed only
                with open(os.path.join(self.dirlog, f),
                          'r', encoding='utf-8') as log:
                    self.logdata[f] = log.read()  # set value
                    self.log_select.InsertItem(index, f)
                index += 1

        if index:
            self.log_select.Focus(selitem)  # make the line the current line
            self.log_select.Select(selitem, on=1)  # default event selection
            self.on_select(self)
    # --------------------------------------------------------------------#

    def on_deselect(self, event):
        """
        Reset on de-selected

        """
        self.textdata.Clear()
        self.selected = None
    # ------------------------------------------------------------------#

    def on_select(self, event):
        """
        show data during items selection

        """
        self.textdata.Clear()  # delete previous append:
        index = self.log_select.GetFocusedItem()
        name = self.log_select.GetItemText(index, 0)
        self.selected = name
        self.textdata.AppendText(self.logdata.get(name))
    # ------------------------------------------------------------------#

    def on_close(self, event):
        """
        Destroy this window
        """
        pub.sendMessage("DESTROY_ORPHANED_WINDOWS", msg='ShowLogs')



# -*- coding: UTF-8 -*-
"""
Name: ydl_preferences.py
Porpose: YouTube Downloader setup dialog
Compatibility: Python3, wxPython Phoenix
Author: Gianluca Pernigotto <jeanlucperni@gmail.com>
Copyleft - 2024 Gianluca Pernigotto <jeanlucperni@gmail.com>
license: GPL3
Rev: Apr.13.2024
Code checker: flake8, pylint

This file is part of Videomass.

   Videomass is free software: you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation, either version 3 of the License, or
   (at your option) any later version.

   Videomass is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with Videomass.  If not, see <http://www.gnu.org/licenses/>.
"""
import os
import webbrowser
import wx
import wx.lib.agw.hyperlink as hpl
from videomass.vdms_sys.settings_manager import ConfigManager


class Ytdlp_Options(wx.Dialog):
    """
    Represents settings and configuration
    storing of the YouTube Downloader.
    """
    # -----------------------------------------------------------------

    def __init__(self, parent):
        """
        self.appdata: (dict) default settings already loaded.
        self.confmanager: instance to ConfigManager class
        self.sett: (dict) current user settings from file conf.

        """
        get = wx.GetApp()
        self.appdata = get.appset
        self.confmanager = ConfigManager(self.appdata['fileconfpath'])
        self.sett = self.confmanager.read_options()

        wx.Dialog.__init__(self, parent, -1, style=wx.DEFAULT_DIALOG_STYLE
                           | wx.RESIZE_BORDER
                           )
        # ----------------------------set notebook
        sizer_base = wx.BoxSizer(wx.VERTICAL)
        notebook = wx.Notebook(self, wx.ID_ANY, style=0)
        sizer_base.Add(notebook, 1, wx.ALL | wx.EXPAND, 5)

        # -----tab 1
        tabOne = wx.Panel(notebook, wx.ID_ANY)
        sizerGen = wx.BoxSizer(wx.VERTICAL)
        sizerGen.Add((0, 10))
        labwa = wx.StaticText(tabOne, wx.ID_ANY, _("Workarounds"))
        sizerGen.Add(labwa, 0, wx.ALL, 5)
        self.ckbx_ssl = wx.CheckBox(tabOne, wx.ID_ANY,
                                    (_('Don’t check SSL certificate'))
                                    )
        self.ckbx_ssl.SetValue(self.appdata['ssl_certificate'])
        sizerGen.Add(self.ckbx_ssl, 0, wx.LEFT, 15)
        sizerGen.Add((0, 10))
        labfn = wx.StaticText(tabOne, wx.ID_ANY, _("Filename options"))
        sizerGen.Add(labfn, 0, wx.ALL, 5)
        self.ckbx_id = wx.CheckBox(tabOne, wx.ID_ANY,
                                   (_('Include the ID in file names')))
        self.ckbx_id.SetValue(self.appdata['include_ID_name'])

        sizerGen.Add(self.ckbx_id, 0, wx.LEFT, 15)

        self.ckbx_limitfn = wx.CheckBox(tabOne, wx.ID_ANY,
                                        (_('Restrict file names to ASCII '
                                           'characters'))
                                        )
        self.ckbx_limitfn.SetValue(self.appdata['restrict_fname'])
        sizerGen.Add(self.ckbx_limitfn, 0, wx.LEFT, 15)
        sizerGen.Add((0, 10))
        labpp = wx.StaticText(tabOne, wx.ID_ANY, _("Post-Process options"))
        sizerGen.Add(labpp, 0, wx.ALL, 5)
        self.ckbx_thumb = wx.CheckBox(tabOne, wx.ID_ANY,
                                      (_('Embed thumbnail in audio file'))
                                      )
        self.ckbx_thumb.SetValue(self.appdata['embed_thumbnails'])
        sizerGen.Add(self.ckbx_thumb, 0, wx.LEFT, 15)
        self.ckbx_meta = wx.CheckBox(tabOne, wx.ID_ANY,
                                     (_('Add metadata to file'))
                                     )
        self.ckbx_meta.SetValue(self.appdata['add_metadata'])
        sizerGen.Add(self.ckbx_meta, 0, wx.LEFT, 15)
        self.ckbx_ow = wx.CheckBox(tabOne, wx.ID_ANY,
                                   (_('Overwrite all files and metadata'))
                                   )
        self.ckbx_ow.SetValue(self.appdata['overwr_dl_files'])
        sizerGen.Add(self.ckbx_ow, 0, wx.LEFT, 15)
        tabOne.SetSizer(sizerGen)
        notebook.AddPage(tabOne, _("General"))

        # -----tab 2
        tabTwo = wx.Panel(notebook, wx.ID_ANY)
        sizerFiles = wx.BoxSizer(wx.VERTICAL)
        sizerFiles.Add((0, 10))
        msg = _("Where do you prefer to save your downloads?")
        labdown = wx.StaticText(tabTwo, wx.ID_ANY, msg)
        sizerFiles.Add(labdown, 0, wx.ALL | wx.EXPAND, 5)
        sizerFiles.Add((0, 10))
        sizeYDLdirdest = wx.BoxSizer(wx.HORIZONTAL)
        sizerFiles.Add(sizeYDLdirdest, 0, wx.EXPAND)
        self.txtctrl_YDLpath = wx.TextCtrl(tabTwo, wx.ID_ANY, "",
                                           style=wx.TE_READONLY
                                           )
        sizeYDLdirdest.Add(self.txtctrl_YDLpath, 1, wx.LEFT | wx.CENTER, 5)
        self.txtctrl_YDLpath.AppendText(self.appdata['ydlp-outputdir'])
        self.btn_YDLpath = wx.Button(tabTwo, wx.ID_ANY, _('Change'))
        sizeYDLdirdest.Add(self.btn_YDLpath, 0, wx.RIGHT | wx.LEFT
                           | wx.ALIGN_CENTER, 5)
        descr = _("Auto-create subdirectories when downloading playlists")
        self.ckbx_playlist = wx.CheckBox(tabTwo, wx.ID_ANY, (descr))
        self.ckbx_playlist.SetValue(self.appdata['playlistsubfolder'])
        sizerFiles.Add(self.ckbx_playlist, 0, wx.ALL, 5)
        tabTwo.SetSizer(sizerFiles)
        notebook.AddPage(tabTwo, _("File Preferences"))

        # -----tab 3
        tabThree = wx.Panel(notebook, wx.ID_ANY)
        sizerextdown = wx.BoxSizer(wx.VERTICAL)
        sizerextdown.Add((0, 10))
        msg = _("Choosing an external downloader")
        labexdtitle = wx.StaticText(tabThree, wx.ID_ANY, msg)
        sizerextdown.Add(labexdtitle, 0, wx.ALL | wx.EXPAND, 5)
        msg = (_("In addition to the default `native` one, yt-dlp can use the "
                 "following external downloaders:\n"
                 "`aria2c`, `avconv`, `axel`, `curl`, `ffmpeg`, `httpie`, "
                 "`wget`.\nPlease note that using an external downloader the "
                 "download status information may not be available."
                 ))
        labdwmsg = wx.StaticText(tabThree, wx.ID_ANY, (msg))
        sizerextdown.Add(labdwmsg, 0, wx.ALL, 5)
        sizerextdown.Add((0, 20))
        msg = _("External downloader executable path")
        labextdw = wx.StaticText(tabThree, wx.ID_ANY, msg)
        sizerextdown.Add(labextdw, 0, wx.LEFT | wx.TOP | wx.BOTTOM, 5)
        exedw = str(self.appdata["external_downloader"])
        self.txtctrl_extdw = wx.TextCtrl(tabThree, wx.ID_ANY, exedw)
        sizerextdown.Add(self.txtctrl_extdw, 0, wx.EXPAND | wx.LEFT | wx.RIGHT
                         | wx.BOTTOM, 5)
        labextdwargs = wx.StaticText(tabThree, wx.ID_ANY,
                                     _("External downloader arguments"))
        sizerextdown.Add(labextdwargs, 0, wx.LEFT | wx.TOP | wx.BOTTOM, 5)
        val = self.appdata["external_downloader_args"]
        args = " ".join(val) if isinstance(val, list) else 'None'
        self.txtctrl_extdw_args = wx.TextCtrl(tabThree, wx.ID_ANY, args)
        sizerextdown.Add(self.txtctrl_extdw_args, 0, wx.EXPAND | wx.LEFT
                         | wx.RIGHT | wx.BOTTOM, 5)
        tabThree.SetSizer(sizerextdown)
        notebook.AddPage(tabThree, _("Download Options"))

        # -----tab 4
        tabFour = wx.Panel(notebook, wx.ID_ANY)
        sizernet = wx.BoxSizer(wx.VERTICAL)
        sizernet.Add((0, 10))
        msg = _("Using a proxy server")
        labproxtitle = wx.StaticText(tabFour, wx.ID_ANY, msg)
        sizernet.Add(labproxtitle, 0, wx.ALL | wx.EXPAND, 5)
        proex = 'socks5://user:pass@127.0.0.1:1080/'
        msg = (_('Use the specified HTTP/HTTPS/SOCKS proxy. To enable SOCKS '
                 'proxy, specify a proper\nscheme, e.g.\t{}\nLeave the text '
                 'field blank for direct connection.').format(proex))
        labproxy = wx.StaticText(tabFour, wx.ID_ANY, (msg))
        sizernet.Add(labproxy, 0, wx.ALL, 5)
        sizernet.Add((0, 20))

        gridproxy = wx.FlexGridSizer(1, 2, 0, 0)
        proxylab = wx.StaticText(tabFour, wx.ID_ANY, _("Proxy"))
        gridproxy.Add(proxylab, 0, wx.ALL | wx.EXPAND, 5)
        self.txtctrl_proxy = wx.TextCtrl(tabFour, wx.ID_ANY, "",
                                         size=(450, -1))
        self.txtctrl_proxy.SetValue(self.appdata["proxy"])
        gridproxy.Add(self.txtctrl_proxy, 0, wx.ALL | wx.EXPAND, 5)
        sizernet.Add(gridproxy, 0, wx.ALL | wx.EXPAND, 5)
        tabFour.SetSizer(sizernet)
        notebook.AddPage(tabFour, _("Network"))
        # -----tab 5
        tabFive = wx.Panel(notebook, wx.ID_ANY)
        sizergeo = wx.BoxSizer(wx.VERTICAL)
        sizergeo.Add((0, 10))
        msg = _("Geo-restriction options")
        labgeoproxtitle = wx.StaticText(tabFive, wx.ID_ANY, msg)
        sizergeo.Add(labgeoproxtitle, 0, wx.ALL | wx.EXPAND, 5)
        msg = (_('Use the following options to try bypassing '
                 'geographic restriction.'))
        labgeogeneral = wx.StaticText(tabFive, wx.ID_ANY, (msg))
        sizergeo.Add(labgeogeneral, 0, wx.ALL, 5)
        sizergeo.Add((0, 20))
        gridgeo = wx.FlexGridSizer(4, 2, 0, 0)
        labgeoproxy = wx.StaticText(tabFive, wx.ID_ANY,
                                    _("Geo verification proxy"))
        gridgeo.Add(labgeoproxy, 0, wx.ALL | wx.EXPAND, 5)
        self.txtctrl_geoproxy = wx.TextCtrl(tabFive, wx.ID_ANY, "",
                                            size=(450, -1))
        tip = _('URL of the proxy to use for IP address verification '
                'on geo-restricted sites.')
        self.txtctrl_geoproxy.SetToolTip(tip)
        self.txtctrl_geoproxy.SetValue(self.appdata["geo_verification_proxy"])
        gridgeo.Add(self.txtctrl_geoproxy, 0, wx.ALL | wx.EXPAND, 5)
        labgeobypass = wx.StaticText(tabFive, wx.ID_ANY, _("Geo bypass"))
        gridgeo.Add(labgeobypass, 0, wx.ALL | wx.EXPAND, 5)
        self.txtctrl_geobypass = wx.TextCtrl(tabFive, wx.ID_ANY, "",
                                             size=(450, -1))
        tip = (_('Bypass geographic restriction via faking X-Forwarded-For '
                 'HTTP header. One of "default" (only when known to be '
                 'useful), or "never"'))
        self.txtctrl_geobypass.SetToolTip(tip)
        self.txtctrl_geobypass.SetValue(self.appdata["geo_bypass"])
        gridgeo.Add(self.txtctrl_geobypass, 0, wx.ALL | wx.EXPAND, 5)
        labgeocountry = wx.StaticText(tabFive, wx.ID_ANY,
                                      _("Geo bypass country"))
        gridgeo.Add(labgeocountry, 0, wx.ALL | wx.EXPAND, 5)
        self.txtctrl_geocountry = wx.TextCtrl(tabFive, wx.ID_ANY, "",
                                              size=(450, -1))
        tip = (_('Two-letter ISO 3166-2 country code that will be used for '
                 'explicit geographic restriction bypassing via faking '
                 'X-Forwarded-For HTTP header.'))
        self.txtctrl_geocountry.SetToolTip(tip)
        self.txtctrl_geocountry.SetValue(self.appdata["geo_bypass_country"])
        gridgeo.Add(self.txtctrl_geocountry, 0, wx.ALL | wx.EXPAND, 5)

        labgeoipblock = wx.StaticText(tabFive, wx.ID_ANY,
                                      _("Geo bypass ip block"))
        gridgeo.Add(labgeoipblock, 0, wx.ALL | wx.EXPAND, 5)
        self.txtctrl_geoipblock = wx.TextCtrl(tabFive, wx.ID_ANY, "",
                                              size=(450, -1))
        tip = (_('IP range in CIDR notation that will be used similarly to '
                 'geo_bypass_country'))
        self.txtctrl_geoipblock.SetToolTip(tip)
        self.txtctrl_geoipblock.SetValue(self.appdata["geo_bypass_ip_block"])
        gridgeo.Add(self.txtctrl_geoipblock, 0, wx.ALL | wx.EXPAND, 5)
        sizergeo.Add(gridgeo, 0, wx.ALL | wx.EXPAND, 5)
        tabFive.SetSizer(sizergeo)
        notebook.AddPage(tabFive, _("Geo-Restriction"))
        # -----tab 6
        tabSix = wx.Panel(notebook, wx.ID_ANY)
        sizerauth = wx.BoxSizer(wx.VERTICAL)
        sizerauth.Add((0, 10))
        msg = _("Authentication with login credentials")
        labauthtitle = wx.StaticText(tabSix, wx.ID_ANY, msg)
        sizerauth.Add(labauthtitle, 0, wx.ALL | wx.EXPAND, 5)
        msg = (_('If you need to login with your credentials please note '
                 'that login with password is not\nsupported for some '
                 'websites (e.g. YouTube, CloudFlare). In this case you can '
                 'include a\ncookie file for authentication (see cookies '
                 'tab).'))
        labauthgen = wx.StaticText(tabSix, wx.ID_ANY, (msg))
        sizerauth.Add(labauthgen, 0, wx.ALL, 5)
        sizerauth.Add((0, 20))
        gridauth = wx.FlexGridSizer(3, 2, 0, 0)
        labuser = wx.StaticText(tabSix, wx.ID_ANY, _("Username"))
        gridauth.Add(labuser, 0, wx.ALL | wx.EXPAND, 5)
        self.txtctrl_username = wx.TextCtrl(tabSix, wx.ID_ANY, "",
                                            size=(300, -1))
        tip = _('Username for authentication purposes')
        self.txtctrl_username.SetToolTip(tip)
        self.txtctrl_username.SetValue(self.appdata["username"])
        gridauth.Add(self.txtctrl_username, 0, wx.ALL | wx.EXPAND, 5)
        labpass = wx.StaticText(tabSix, wx.ID_ANY, _("Password"))
        gridauth.Add(labpass, 0, wx.ALL | wx.EXPAND, 5)
        self.txtctrl_pass = wx.TextCtrl(tabSix, wx.ID_ANY, "",
                                        style=wx.TE_PASSWORD, size=(300, -1))
        tip = _('Password for authentication purposes')
        self.txtctrl_pass.SetToolTip(tip)
        self.txtctrl_pass.SetValue(self.appdata.get("password", ''))
        gridauth.Add(self.txtctrl_pass, 0, wx.ALL | wx.EXPAND, 5)

        labvpass = wx.StaticText(tabSix, wx.ID_ANY, _("Video Password"))
        gridauth.Add(labvpass, 0, wx.ALL | wx.EXPAND, 5)
        self.txtctrl_vpass = wx.TextCtrl(tabSix, wx.ID_ANY, "",
                                         style=wx.TE_PASSWORD, size=(300, -1))
        tip = _('Password for accessing a video')
        self.txtctrl_vpass.SetToolTip(tip)
        self.txtctrl_vpass.SetValue(self.appdata.get("videopassword", ''))
        gridauth.Add(self.txtctrl_vpass, 0, wx.ALL | wx.EXPAND, 5)
        sizerauth.Add(gridauth, 0, wx.ALL | wx.EXPAND, 5)
        tabSix.SetSizer(sizerauth)
        notebook.AddPage(tabSix, _("Authentication"))

        # -----tab 7
        tabSev = wx.Panel(notebook, wx.ID_ANY)
        sizercook = wx.BoxSizer(wx.VERTICAL)
        sizercook.Add((0, 10))
        msg = _("Using cookies to gain access to websites")
        labcooktitle = wx.StaticText(tabSev, wx.ID_ANY, msg)
        sizercook.Add(labcooktitle, 0, wx.ALL | wx.EXPAND, 5)
        msg = (_("Here you can pass your `Netscape HTTP Cookie File` in order "
                 "to access websites that require authentication.\nIf you "
                 "don't know how to obtain this type of file, you can let "
                 "the application try to extract it from "
                 "your browser\nautomatically. The currently supported "
                 "browsers are: brave, chrome, chromium, edge, firefox, "
                 "opera, safari, vivaldi.\n\nFor more information please "
                 "consult the documentation at the following link:"))
        labcookgen = wx.StaticText(tabSev, wx.ID_ANY, (msg))
        sizercook.Add(labcookgen, 0, wx.ALL, 5)
        url1 = ("https://github.com/yt-dlp/yt-dlp/wiki/"
                "FAQ#how-do-i-pass-cookies-to-yt-dlp")
        link1 = hpl.HyperLinkCtrl(tabSev, -1, url1, URL=(url1))
        sizercook.Add(link1, 0, wx.LEFT | wx.EXPAND, 15)
        sizercook.Add((0, 20))
        sizercookie = wx.BoxSizer(wx.HORIZONTAL)

        msg = _('Allow cookies for authentication')
        self.ckbx_usecook = wx.CheckBox(tabSev, wx.ID_ANY, msg)
        self.ckbx_usecook.SetValue(self.appdata['use_cookie_file'])
        sizercook.Add(self.ckbx_usecook, 0, wx.LEFT, 5)
        labcookie = wx.StaticText(tabSev, wx.ID_ANY, _("Cookie file"))
        sizercookie.Add(labcookie, 0, wx.LEFT | wx.TOP | wx.EXPAND, 5)
        self.txtctrl_cook = wx.TextCtrl(tabSev, wx.ID_ANY, "",
                                        style=wx.TE_READONLY
                                        )
        self.txtctrl_cook.SetValue(self.appdata["cookiefile"])
        tip = _('File name from where cookies should be read and dumped to')
        self.txtctrl_cook.SetToolTip(tip)
        sizercookie.Add(self.txtctrl_cook, 1, wx.ALL | wx.EXPAND, 5)
        self.btn_cookie = wx.Button(tabSev, wx.ID_ANY, _('Browse'))
        sizercookie.Add(self.btn_cookie, 0, wx.RIGHT | wx.ALIGN_CENTER, 5)
        sizercook.Add(sizercookie, 0, wx.ALL | wx.EXPAND, 5)
        msg = _('Extract cookies automatically from your browser')
        self.ckbx_autocook = wx.CheckBox(tabSev, wx.ID_ANY, msg)
        self.ckbx_autocook.SetValue(self.appdata['autogen_cookie_file'])
        sizercook.Add(self.ckbx_autocook, 0, wx.ALL, 5)
        sizerautocook = wx.BoxSizer(wx.HORIZONTAL)
        self.labautocook = wx.StaticText(tabSev, wx.ID_ANY,
                                         _("Web Browser"))
        sizerautocook.Add(self.labautocook, 0, wx.LEFT | wx.TOP | wx.CENTER, 5)
        webbrowser = ('brave', 'chrome', 'chromium', 'edge',
                      'firefox', 'opera', 'safari', 'vivaldi')
        self.cmbx_browser = wx.ComboBox(tabSev, wx.ID_ANY,
                                        choices=webbrowser,
                                        size=(-1, -1),
                                        style=wx.CB_DROPDOWN | wx.CB_READONLY
                                        )
        self.cmbx_browser.SetValue(self.appdata['webbrowser'])
        sizerautocook.Add(self.cmbx_browser, 0, wx.LEFT | wx.TOP
                          | wx.CENTRE, 5)
        if not self.appdata['autogen_cookie_file']:
            self.labautocook.Disable(), self.cmbx_browser.Disable()
        if not self.sett['use_cookie_file']:
            self.btn_cookie.Disable(), self.ckbx_autocook.Disable()
            self.txtctrl_cook.Disable()
        sizercook.Add(sizerautocook, 0, wx.LEFT | wx.EXPAND, 5)
        tabSev.SetSizer(sizercook)
        notebook.AddPage(tabSev, _("Cookies"))
        # ----- confirm buttons section
        grdBtn = wx.GridSizer(1, 2, 0, 0)
        grdhelp = wx.GridSizer(1, 1, 0, 0)
        btn_help = wx.Button(self, wx.ID_HELP, "")
        grdhelp.Add(btn_help, 0, wx.ALL | wx.ALIGN_CENTER_VERTICAL, 5)
        grdBtn.Add(grdhelp)
        grdexit = wx.BoxSizer(wx.HORIZONTAL)
        btn_cancel = wx.Button(self, wx.ID_CANCEL, "")
        grdexit.Add(btn_cancel, 0, wx.ALIGN_CENTER_VERTICAL)
        btn_ok = wx.Button(self, wx.ID_OK, "")
        grdexit.Add(btn_ok, 0, wx.LEFT | wx.ALIGN_CENTER_VERTICAL, 5)
        grdBtn.Add(grdexit, flag=wx.ALL | wx.ALIGN_RIGHT | wx.RIGHT, border=5)
        sizer_base.Add(grdBtn, 0, wx.EXPAND)

        # ----- Properties
        if self.appdata['ostype'] == 'Darwin':
            labdown.SetFont(wx.Font(13, wx.DEFAULT, wx.NORMAL, wx.BOLD))
            labdwmsg.SetFont(wx.Font(11, wx.SWISS, wx.NORMAL, wx.NORMAL))
            labexdtitle.SetFont(wx.Font(13, wx.DEFAULT, wx.NORMAL, wx.BOLD))
            labproxtitle.SetFont(wx.Font(13, wx.DEFAULT, wx.NORMAL, wx.BOLD))
            labproxy.SetFont(wx.Font(11, wx.SWISS, wx.NORMAL, wx.NORMAL))
            labgeoproxtitle.SetFont(wx.Font(13, wx.DEFAULT, wx.NORMAL,
                                            wx.BOLD))
            labgeogeneral.SetFont(wx.Font(11, wx.SWISS, wx.NORMAL, wx.NORMAL))
            labauthtitle.SetFont(wx.Font(13, wx.DEFAULT, wx.NORMAL, wx.BOLD))
            labauthgen.SetFont(wx.Font(11, wx.SWISS, wx.NORMAL, wx.NORMAL))
            labcooktitle.SetFont(wx.Font(13, wx.DEFAULT, wx.NORMAL, wx.BOLD))
            labcookgen.SetFont(wx.Font(11, wx.SWISS, wx.NORMAL, wx.NORMAL))
        else:
            labdown.SetFont(wx.Font(10, wx.DEFAULT, wx.NORMAL, wx.BOLD))
            labdwmsg.SetFont(wx.Font(8, wx.SWISS, wx.NORMAL, wx.NORMAL))
            labexdtitle.SetFont(wx.Font(10, wx.DEFAULT, wx.NORMAL, wx.BOLD))
            labproxtitle.SetFont(wx.Font(10, wx.DEFAULT, wx.NORMAL, wx.BOLD))
            labproxy.SetFont(wx.Font(8, wx.SWISS, wx.NORMAL, wx.NORMAL))
            labgeoproxtitle.SetFont(wx.Font(10, wx.DEFAULT, wx.NORMAL,
                                            wx.BOLD))
            labgeogeneral.SetFont(wx.Font(8, wx.SWISS, wx.NORMAL, wx.NORMAL))
            labauthtitle.SetFont(wx.Font(10, wx.DEFAULT, wx.NORMAL, wx.BOLD))
            labauthgen.SetFont(wx.Font(8, wx.SWISS, wx.NORMAL, wx.NORMAL))
            labcooktitle.SetFont(wx.Font(10, wx.DEFAULT, wx.NORMAL, wx.BOLD))
            labcookgen.SetFont(wx.Font(8, wx.SWISS, wx.NORMAL, wx.NORMAL))

        self.SetTitle(_("YouTube Downloader Options"))

        # ------ set sizer
        self.SetMinSize((700, 550))
        self.SetSizer(sizer_base)
        sizer_base.Fit(self)
        self.Layout()

        # ----------------------Binding (EVT)----------------------#
        self.Bind(wx.EVT_BUTTON, self.on_downloadfile, self.btn_YDLpath)
        self.Bind(wx.EVT_BUTTON, self.on_help, btn_help)
        self.Bind(wx.EVT_BUTTON, self.on_cancel, btn_cancel)
        self.Bind(wx.EVT_BUTTON, self.on_ok, btn_ok)
        self.Bind(wx.EVT_TEXT, self.on_ext_downl, self.txtctrl_extdw)
        self.Bind(wx.EVT_TEXT, self.on_ext_downl_args, self.txtctrl_extdw_args)
        self.Bind(wx.EVT_BUTTON, self.on_cookie_file, self.btn_cookie)
        self.Bind(wx.EVT_CHECKBOX, self.on_autogen_cookie, self.ckbx_autocook)
        self.Bind(wx.EVT_COMBOBOX, self.on_autogen_cookie, self.cmbx_browser)
        self.Bind(wx.EVT_CHECKBOX, self.on_enable_cookie, self.ckbx_usecook)
        # --------------------------------------------#

    def on_enable_cookie(self, event):
        """
        Event triggered on enabling cookies CheckBox.
        """
        if self.ckbx_usecook.GetValue():
            self.ckbx_autocook.Enable()
            self.btn_cookie.Enable()
            self.txtctrl_cook.Enable()
            self.sett['use_cookie_file'] = True
        else:
            ckf = self.txtctrl_cook.GetValue()
            if os.path.exists(ckf) and os.path.isfile(ckf):
                if wx.MessageBox(_("Do you want to remove the current "
                                   "cookie file from your system?"),
                                 _('Please confirm'), wx.ICON_QUESTION
                                 | wx.CANCEL | wx.YES_NO, self) == wx.YES:
                    os.remove(ckf)
            self.ckbx_autocook.SetValue(False)
            self.on_autogen_cookie(None)
            self.ckbx_autocook.Disable()
            self.btn_cookie.Disable()
            self.txtctrl_cook.SetValue("")
            self.txtctrl_cook.Disable()
            self.sett['use_cookie_file'] = False
    # ---------------------------------------------------------------------#

    def on_autogen_cookie(self, event):
        """
        Event on checking Generate a cookie file from web
        browser CheckBox.
        """
        if self.ckbx_autocook.GetValue():
            self.labautocook.Enable(), self.cmbx_browser.Enable()
            self.sett['autogen_cookie_file'] = True
            self.sett['webbrowser'] = self.cmbx_browser.GetStringSelection()
            path = os.path.join(self.appdata["confdir"], 'cookies.txt')
            self.sett["cookiefile"] = path
            self.txtctrl_cook.SetValue(path)
            self.sett["cookiesfrombrowser"] = (self.sett['webbrowser'],
                                               None, None, None)

        else:
            self.labautocook.Disable(), self.cmbx_browser.Disable()
            self.sett['autogen_cookie_file'] = False
            self.sett["cookiefile"] = ""
            self.txtctrl_cook.SetValue("")
            self.sett["cookiesfrombrowser"] = (None, None, None, None)
    # ---------------------------------------------------------------------#

    def on_web_browser(self, event):
        """
        Event on choice a web browser item from ComboBox
        """
        self.sett['webbrowser'] = self.cmbx_browser.GetStringSelection()
        self.sett["cookiesfrombrowser"] = (self.sett['webbrowser'],
                                           None, None, None)
    # ---------------------------------------------------------------------#

    def on_cookie_file(self, event):
        """
        Event triggered clicking on Button to
        select a new cookies.txt file.
        """
        fmt = ('*cookies.txt')
        wild = f"Netscape HTTP Cookie File ({fmt})|{fmt}"
        with wx.FileDialog(self, _("Locate your cookie file"),
                           defaultDir=os.path.expanduser('~'),
                           wildcard=wild,
                           style=wx.FD_OPEN | wx.FD_FILE_MUST_EXIST) as fdlg:
            if fdlg.ShowModal() == wx.ID_CANCEL:
                return
            filename = fdlg.GetPath()
            self.txtctrl_cook.SetValue(filename)
    # ---------------------------------------------------------------------#

    def on_downloadfile(self, event):
        """
        Event clicking on Button to setting up a custom
        path where save downloads.
        """
        dlg = wx.DirDialog(self, _("Choose Destination"),
                           self.appdata['ydlp-outputdir'], wx.DD_DEFAULT_STYLE
                           )
        if dlg.ShowModal() == wx.ID_OK:
            self.txtctrl_YDLpath.Clear()
            getpath = self.appdata['getpath'](dlg.GetPath())
            self.txtctrl_YDLpath.AppendText(getpath)
            self.sett['ydlp-outputdir'] = getpath
            dlg.Destroy()
    # ---------------------------------------------------------------------#

    def on_ext_downl(self, event):
        """
        Event on entering executable path of external downloader
        """
        val = str(self.txtctrl_extdw.GetValue()).strip()
        downloader = None if val in ("", "None", "none") else val
        self.sett["external_downloader"] = downloader
    # -------------------------------------------------------------------#

    def on_ext_downl_args(self, event):
        """
        Event on entering arguments for external downloader
        """
        val = str(self.txtctrl_extdw_args.GetValue()).strip()
        args = None if val in ("", "None", "none") else val.split()
        self.sett["external_downloader_args"] = args
    # -------------------------------------------------------------------#

    def on_help(self, event):
        """
        Open default web browser via Python Web-browser controller.
        see <https://docs.python.org/3.8/library/webbrowser.html>
        """
        page = ('https://jeanslack.github.io/Videomass/User-guide/'
                'YouTube_Downloader_Options_en.pdf')
        webbrowser.open(page)
    # --------------------------------------------------------------------#

    def on_cancel(self, event):
        """
        Close event
        """
        event.Skip()
    # --------------------------------------------------------------------#

    def on_ok(self, event):
        """
        Writes the new changes to configuration file
        aka `settings.json` and updates `appdata` dict.
        """
        if not self.sett['trashdir_loc'].strip():
            self.sett['trashdir_loc'] = self.appdata['trashdir_default']
        self.sett['username'] = self.txtctrl_username.GetValue()
        self.sett['add_metadata'] = self.ckbx_meta.GetValue()
        self.sett['embed_thumbnails'] = self.ckbx_thumb.GetValue()
        self.sett['playlistsubfolder'] = self.ckbx_playlist.GetValue()
        self.sett['ssl_certificate'] = self.ckbx_ssl.GetValue()
        self.sett['overwr_dl_files'] = self.ckbx_ow.GetValue()
        self.sett['include_ID_name'] = self.ckbx_id.GetValue()
        self.sett['restrict_fname'] = self.ckbx_limitfn.GetValue()
        self.sett['proxy'] = self.txtctrl_proxy.GetValue()
        self.sett['geo_verification_proxy'] = self.txtctrl_geoproxy.GetValue()
        self.sett['geo_bypass'] = self.txtctrl_geobypass.GetValue()
        self.sett['geo_bypass_country'] = self.txtctrl_geocountry.GetValue()
        self.sett['geo_bypass_ip_block'] = self.txtctrl_geoipblock.GetValue()
        self.sett['cookiefile'] = self.txtctrl_cook.GetValue()
        self.confmanager.write_options(**self.sett)
        self.appdata.update(self.sett)
        # do not store this data in the configuration file
        self.appdata['password'] = self.txtctrl_pass.GetValue()
        self.appdata['videopassword'] = self.txtctrl_vpass.GetValue()
        event.Skip()



# -*- coding: UTF-8 -*-

"""
Name: ffplay_file.py
Porpose: playback media file via ffplay
Compatibility: Python3, wxPython Phoenix
Author: Gianluca Pernigotto <jeanlucperni@gmail.com>
Copyleft - 2024 Gianluca Pernigotto <jeanlucperni@gmail.com>
license: GPL3
Rev: Apr.20.2024
Code checker: flake8, pylint

This file is part of Videomass.

   Videomass is free software: you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation, either version 3 of the License, or
   (at your option) any later version.

   Videomass is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with Videomass.  If not, see <http://www.gnu.org/licenses/>.
"""
import os
from threading import Thread
import subprocess
import platform
import wx
from videomass.vdms_io.make_filelog import make_log_template
if not platform.system() == 'Windows':
    import shlex


def msg_error(msg):
    """
    Receive error messages via wxCallafter
    """
    wx.MessageBox(f"FFplay ERROR:  {msg}",
                  _('Videomass - Error!'), wx.ICON_ERROR)


def msg_info(msg):
    """
    Receive info messages via wxCallafter
    """
    wx.MessageBox(f"FFplay:  {msg}", "Videomass", wx.ICON_INFORMATION)


class FilePlay(Thread):
    """
    Playback local file with ffplay media player via subprocess.Popen
    class (ffplay is a player which need x-window-terminal-emulator)

    """

    def __init__(self, filepath, timeseq, param, autoexit):
        """
        The self.FFPLAY_loglevel has flag 'error -hide_banner' by default,
        see videomass.conf for details.
        WARNING Do not use the "-stats" option as it does not work here.

        """
        get = wx.GetApp()  # get videomass wx.App attribute
        self.appdata = get.appset
        self.filename = filepath  # file name selected
        self.time_seq = timeseq  # seeking
        self.param = param  # additional parameters if present
        self.autoexit = '-autoexit' if autoexit else ''
        self.logf = os.path.join(self.appdata['logdir'], 'ffplay.log')
        make_log_template('ffplay.log', self.appdata['logdir'], mode="w")
        # set initial file LOG

        Thread.__init__(self)
        self.start()
    # ----------------------------------------------------------------#

    def run(self):
        """
        Get and redirect output errors on proc.returncode instance and on
        OSError exception. Otherwise the getted output as information
        given by error[1]

        """
        # time.sleep(.5)
        cmd = " ".join(f'"{self.appdata["ffplay_cmd"]}" '
                       f'{self.appdata["ffplay-default-args"]} '
                       f'{self.appdata["ffplay_loglev"]} '
                       f'{self.autoexit} '
                       f'{self.time_seq[0]} '
                       f'-i "{self.filename}" '
                       f'{self.time_seq[1]} '
                       f'{self.param}'.split()
                       )
        self.logwrite(cmd)

        if not platform.system() == 'Windows':
            cmd = shlex.split(cmd)
            info = None
            shell = False
        else:
            # NOTE: on MS-Windows, 'info' flag do not work with ffplay
            # is Fixed with shell=True flag.
            shell = True
            info = None
            # info = subprocess.STARTUPINFO()
            # info.dwFlags |= subprocess.SW_HIDE
        try:
            with subprocess.Popen(cmd,
                                  shell=shell,
                                  stderr=subprocess.PIPE,
                                  universal_newlines=True,
                                  encoding=self.appdata['encoding'],
                                  startupinfo=info,
                                  ) as proc:
                error = proc.communicate()[1]

                if error:  # ffplay error
                    wx.CallAfter(msg_info, error)
                    self.logerror(error)  # append log error
                    return

        except OSError as err:  # subprocess error
            wx.CallAfter(msg_error, err)
            self.logerror(err)  # append log error
            return
    # ----------------------------------------------------------------#

    def logwrite(self, cmd):
        """
        write ffplay command log
        """
        with open(self.logf, "a", encoding='utf-8') as log:
            log.write(f"{cmd}\n")
    # ----------------------------------------------------------------#

    def logerror(self, error):
        """
        write ffplay errors
        """
        with open(self.logf, "a", encoding='utf-8') as logerr:
            logerr.write(f"\n[FFMPEG] FFplay "
                         f"OUTPUT:\n{error}\n")



# -*- coding: UTF-8 -*-
"""
FileName: av_conversions.py
Porpose: audio/video conversions interface
Compatibility: Python3, wxPython4 Phoenix
Author: Gianluca Pernigotto <jeanlucperni@gmail.com>
Copyleft - 2024 Gianluca Pernigotto <jeanlucperni@gmail.com>
license: GPL3
Rev: Mar.22.2024
Code checker: flake8, pylint

This file is part of Videomass.

   Videomass is free software: you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation, either version 3 of the License, or
   (at your option) any later version.

   Videomass is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with Videomass.  If not, see <http://www.gnu.org/licenses/>.
"""
import os
import sys
import wx
import wx.lib.scrolledpanel as scrolled
from pubsub import pub
from videomass.vdms_utils.utils import update_timeseq_duration
from videomass.vdms_utils.get_bmpfromsvg import get_bmp
from videomass.vdms_io.io_tools import stream_play
from videomass.vdms_io.checkup import check_files
from videomass.vdms_dialogs.epilogue import Formula
from videomass.vdms_dialogs import setting_profiles
from videomass.vdms_dialogs.filter_crop import Crop
from videomass.vdms_dialogs.filter_transpose import Transpose
from videomass.vdms_dialogs.filter_denoisers import Denoisers
from videomass.vdms_dialogs.filter_deinterlace import Deinterlace
from videomass.vdms_dialogs.filter_scale import Scale
from videomass.vdms_dialogs.filter_stab import VidstabSet
from videomass.vdms_dialogs.filter_colorcorrection import ColorEQ
from . video_encoders.video_no_enc import Video_No_Enc
from . video_encoders.mpeg4 import Mpeg_4
from . video_encoders.av1_aom import AV1_Aom
from . video_encoders.av1_svt import AV1_Svt
from . video_encoders.vp9_webm import Vp9_WebM
from . video_encoders.avc_x264 import Avc_X264
from . video_encoders.hevc_x265 import Hevc_X265
from . video_encoders.video_encodercopy import Copy_Vcodec
from . audio_encoders.acodecs import AudioEncoders
from . miscellaneous.miscell import Miscellaneous


class AV_Conv(wx.Panel):
    """
    Panel GUI for audio and video conversions
    """
    # colour rappresentetion in html
    AZURE = '#15a6a6'
    YELLOW = '#bd9f00'
    RED = '#ea312d'
    ORANGE = '#f28924'
    GREENOLIVE = '#6aaf23'
    GREEN = '#268826'
    CYAN = '#61ccc7'  # rgb form (wx.Colour(97, 204, 199)
    VIOLET = '#D64E93'
    LIMEGREEN = '#87A615'
    TROPGREEN = '#15A660'
    WHITE = '#fbf4f4'
    BLACK = '#060505'

    # MUXERS dictionary:
    MUXERS = {'mkv': 'matroska', 'avi': 'avi', 'mp4': 'mp4',
              'm4v': 'null', 'ogg': 'ogg', 'webm': 'webm',
              }
    # Namings in the video container selection combo box:
    VCODECS = ({"MPEG-4": {"-c:v mpeg4": ["avi"]},
                "XVID MPEG-4": {"-c:v libxvid": ["avi"]},
                "H.264": {"-c:v libx264": ["mkv", "mp4", "avi", "m4v"]},
                "H.264 10-bit": {"-c:v libx264": ["mkv", "mp4", "avi", "m4v"]},
                "H.265": {"-c:v libx265": ["mkv", "mp4", "avi", "m4v"]},
                "H.265 10-bit": {"-c:v libx265": ["mkv", "mp4", "avi", "m4v"]},
                "AOM-AV1": {"-c:v libaom-av1": ["mkv", "webm", "mp4"]},
                "SVT-AV1": {"-c:v libsvtav1": ["mkv", "webm"]},
                "SVT-AV1 10-bit": {"-c:v libsvtav1": ["mkv", "webm"]},
                "VP9": {"-c:v libvpx-vp9": ["webm", "mkv"]},
                "Copy": {"-c:v copy": ["mkv", "mp4", "avi", "m4v", "ogv",
                                       "webm", "Copy"]}
                })
    # Namings in the audio format selection on Container combobox:
    A_FORMATS = ('wav', 'mp3', 'ac3', 'ogg', 'flac', 'm4a', 'aac', 'opus')
    # ------------------------------------------------------------------#

    def __init__(self, parent):
        """
        Collects all the values of the
        GUI controls used in this panel
        """
        self.parent = parent  # parent is the MainFrame
        get = wx.GetApp()  # get data from bootstrap
        self.appdata = get.appset
        icons = get.iconset
        self.videopanel = None

        if 'wx.svg' in sys.modules:  # only available in wx version 4.1 to up
            bmpplay = get_bmp(icons['preview'], ((16, 16)))
            self.bmpreset = get_bmp(icons['clear'], ((16, 16)))
            bmpresize = get_bmp(icons['scale'], ((16, 16)))
            bmpcrop = get_bmp(icons['crop'], ((16, 16)))
            bmprotate = get_bmp(icons['rotate'], ((16, 16)))
            bmpdeinterlace = get_bmp(icons['deinterlace'], ((16, 16)))
            bmpdenoiser = get_bmp(icons['denoiser'], ((16, 16)))
            bmpstab = get_bmp(icons['stabilizer'], ((16, 16)))
            bmpsaveprf = get_bmp(icons['addtoprst'], ((16, 16)))
            bmpcoloreq = get_bmp(icons['coloreq'], ((16, 16)))
        else:
            bmpplay = wx.Bitmap(icons['preview'], wx.BITMAP_TYPE_ANY)
            self.bmpreset = wx.Bitmap(icons['clear'], wx.BITMAP_TYPE_ANY)
            bmpresize = wx.Bitmap(icons['scale'], wx.BITMAP_TYPE_ANY)
            bmpcrop = wx.Bitmap(icons['crop'], wx.BITMAP_TYPE_ANY)
            bmprotate = wx.Bitmap(icons['rotate'], wx.BITMAP_TYPE_ANY)
            bmpdeinterlace = wx.Bitmap(icons['deinterlace'],
                                       wx.BITMAP_TYPE_ANY)
            bmpdenoiser = wx.Bitmap(icons['denoiser'], wx.BITMAP_TYPE_ANY)
            bmpstab = wx.Bitmap(icons['stabilizer'], wx.BITMAP_TYPE_ANY)
            bmpsaveprf = wx.Bitmap(icons['addtoprst'], wx.BITMAP_TYPE_ANY)
            bmpcoloreq = wx.Bitmap(icons['coloreq'], wx.BITMAP_TYPE_ANY)

        # Default keys:values dictionary definition in this class
        self.opt = {"Media": "Video", "VidCmbxStr": "H.264",
                    "OutputFormat": "mkv", "VideoCodec": "-c:v libx264",
                    "ext_input": "", "Passes": "Auto", "InputDir": "",
                    "OutputDir": "", "SubtitleMap": "-map 0:s?",
                    "Deinterlace": "", "Interlace": "", "ColorEQ": "",
                    "PixelFormat": "", "Orientation": ["", ""], "Crop": "",
                    "CropColor": "", "Scale": "", "Setdar": "", "Setsar": "",
                    "Denoiser": "", "Vidstabtransform": "",
                    "Vidstabdetect": "", "Unsharp": "", "Makeduo": False,
                    "VFilters": "", "CmdVideoParams": "", "CmdAudioParams": "",
                    "VideoMap": "-map 0:v?",

                    }
        wx.Panel.__init__(self, parent, -1)
        # ------------ widgets
        sizer_base = wx.BoxSizer(wx.VERTICAL)
        # ------------------ BEGIN BOX top
        sizer_base.Add(10, 10)
        sizer_convin = wx.BoxSizer(wx.HORIZONTAL)
        txtmedia = wx.StaticText(self, wx.ID_ANY, _('Media:'))
        sizer_convin.Add(txtmedia, 0, wx.LEFT | wx.CENTRE, 5)
        self.cmb_Media = wx.ComboBox(self, wx.ID_ANY,
                                     choices=['Video', 'Audio'],
                                     size=(100, -1), style=wx.CB_DROPDOWN
                                     | wx.CB_READONLY
                                     )
        sizer_convin.Add(self.cmb_Media, 0, wx.LEFT | wx.CENTRE, 5)
        txtVcod = wx.StaticText(self, wx.ID_ANY, 'Video Encoder:')
        sizer_convin.Add(txtVcod, 0, wx.LEFT | wx.CENTRE, 20)
        self.cmb_vencoder = wx.ComboBox(self, wx.ID_ANY,
                                        choices=list(AV_Conv.VCODECS.keys()),
                                        size=(120, -1),
                                        style=wx.CB_DROPDOWN | wx.CB_READONLY
                                        )
        sizer_convin.Add(self.cmb_vencoder, 0, wx.LEFT | wx.CENTRE, 5)
        txtFormat = wx.StaticText(self, wx.ID_ANY, _('Container:'))
        sizer_convin.Add(txtFormat, 0, wx.LEFT | wx.CENTRE, 20)
        choices = list(AV_Conv.VCODECS['H.264'].values())[0]
        self.cmb_cont = wx.ComboBox(self, wx.ID_ANY,
                                    choices=choices,
                                    size=(100, -1),
                                    style=wx.CB_DROPDOWN
                                    | wx.CB_READONLY,
                                    )
        sizer_convin.Add(self.cmb_cont, 0, wx.LEFT | wx.CENTRE, 5)
        self.btn_saveprst = wx.Button(self, wx.ID_ANY,
                                      _("Save Preset"), size=(-1, -1))
        self.btn_saveprst.SetBitmap(bmpsaveprf, wx.LEFT)
        sizer_convin.Add(self.btn_saveprst, 0, wx.LEFT | wx.CENTRE, 20)
        msg = _("Target")
        box1 = wx.StaticBox(self, wx.ID_ANY, msg)
        box_convin = wx.StaticBoxSizer(box1, wx.HORIZONTAL)
        box_convin.Add(sizer_convin, 0, wx.ALL | wx.CENTRE, 5)
        sizer_base.Add(box_convin, 0, wx.BOTTOM | wx.CENTRE, 5)
        # END BOX top Media and Format

        # ------------------ BEGIN NOTEBOOK CONSTRUCTOR
        self.notebook = wx.Notebook(self, wx.ID_ANY,
                                    style=wx.NB_NOPAGETHEME | wx.NB_TOP
                                    )
        sizer_base.Add(self.notebook, 1, wx.ALL | wx.EXPAND, 5)
        # -------------- BEGIN NOTEBOOK PANEL 1
        self.nb_Video = wx.Panel(self.notebook, wx.ID_ANY)
        sizer_nbVideo = wx.BoxSizer(wx.HORIZONTAL)
        # VIDEO BOX video encoders central box
        box3 = wx.StaticBox(self.nb_Video, wx.ID_ANY, "")
        box_opt = wx.StaticBoxSizer(box3, wx.VERTICAL)
        sizer_nbVideo.Add(box_opt, 1, wx.ALL | wx.EXPAND, 5)

        # Inizializing VIDEO panels
        self.mpeg4panel = Mpeg_4(self.nb_Video, self.opt)
        self.aompanel = AV1_Aom(self.nb_Video, self.opt)
        self.svtpanel = AV1_Svt(self.nb_Video, self.opt)
        self.vp9panel = Vp9_WebM(self.nb_Video, self.opt)
        self.h264panel = Avc_X264(self.nb_Video, self.opt)
        self.h265panel = Hevc_X265(self.nb_Video, self.opt)
        self.vcopypanel = Copy_Vcodec(self.nb_Video, self.opt)
        self.disablevidpanels = Video_No_Enc(self.nb_Video)

        for vpan in (self.aompanel, self.svtpanel, self.vp9panel,
                     self.mpeg4panel, self.h264panel, self.h265panel,
                     self.vcopypanel, self.disablevidpanels,
                     ):
            box_opt.Add(vpan, 0, wx.ALL | wx.EXPAND, 5)

        self.vp9panel.Hide(), self.h265panel.Hide()
        self.aompanel.Hide(), self.svtpanel.Hide()
        self.vcopypanel.Hide(), self.mpeg4panel.Hide()
        self.disablevidpanels.Hide()
        self.videopanel = self.h264panel
        self.videopanel.Show()

        # BOX Video filters
        box4 = wx.StaticBox(self.nb_Video, wx.ID_ANY, _("Video Filters"))
        self.box_Vfilters = wx.StaticBoxSizer(box4, wx.VERTICAL)
        self.btn_preview = wx.Button(self.nb_Video, wx.ID_ANY,
                                     _("Preview"), size=(-1, -1))
        self.btn_preview.SetBitmap(bmpplay, wx.LEFT)

        self.box_Vfilters.Add(self.btn_preview, 0, wx.ALL | wx.EXPAND, 5)
        self.btn_preview.Disable()
        self.btn_reset = wx.Button(self.nb_Video, wx.ID_ANY,
                                   _("Reset all"), size=(-1, -1))
        self.btn_reset.SetBitmap(self.bmpreset, wx.LEFT)
        self.box_Vfilters.Add(self.btn_reset, 0, wx.ALL | wx.EXPAND, 5)
        self.btn_reset.Disable()
        lineflt = wx.StaticLine(self.nb_Video,
                                wx.ID_ANY,
                                pos=wx.DefaultPosition,
                                size=wx.DefaultSize,
                                style=wx.LI_HORIZONTAL,
                                name=wx.StaticLineNameStr,
                                )
        self.box_Vfilters.Add(lineflt, 0, wx.ALL | wx.EXPAND, 10)
        sizer_nbVideo.Add(self.box_Vfilters, 0, wx.ALL | wx.EXPAND, 5)
        self.filterVpanel = scrolled.ScrolledPanel(self.nb_Video, -1,
                                                   size=(180, 700),
                                                   style=wx.TAB_TRAVERSAL
                                                   | wx.BORDER_NONE,
                                                   name="panelscroll",
                                                   )
        sizer_Vfilter = wx.BoxSizer(wx.VERTICAL)
        self.btn_videosize = wx.Button(self.filterVpanel, wx.ID_ANY,
                                       _("Resize"), size=(-1, -1))
        self.btn_videosize.SetBitmap(bmpresize, wx.LEFT)
        sizer_Vfilter.Add(self.btn_videosize, 0, wx.ALL | wx.EXPAND, 5)
        self.btn_crop = wx.Button(self.filterVpanel, wx.ID_ANY,
                                  _("Crop"), size=(-1, -1))
        self.btn_crop.SetBitmap(bmpcrop, wx.LEFT)
        sizer_Vfilter.Add(self.btn_crop, 0, wx.ALL | wx.EXPAND, 5)
        self.btn_rotate = wx.Button(self.filterVpanel, wx.ID_ANY,
                                    _("Transpose"), size=(-1, -1))
        self.btn_rotate.SetBitmap(bmprotate, wx.LEFT)

        sizer_Vfilter.Add(self.btn_rotate, 0, wx.ALL | wx.EXPAND, 5)
        self.btn_lacing = wx.Button(self.filterVpanel, wx.ID_ANY,
                                    _("Deinterlace"), size=(-1, -1))
        self.btn_lacing.SetBitmap(bmpdeinterlace, wx.LEFT)
        sizer_Vfilter.Add(self.btn_lacing, 0, wx.ALL | wx.EXPAND, 5)
        self.btn_denois = wx.Button(self.filterVpanel, wx.ID_ANY,
                                    _("Denoise"), size=(-1, -1))
        self.btn_denois.SetBitmap(bmpdenoiser, wx.LEFT)
        sizer_Vfilter.Add(self.btn_denois, 0, wx.ALL | wx.EXPAND, 5)
        self.btn_vidstab = wx.Button(self.filterVpanel, wx.ID_ANY,
                                     _("Stabilize"), size=(-1, -1))
        self.btn_vidstab.SetBitmap(bmpstab, wx.LEFT)
        sizer_Vfilter.Add(self.btn_vidstab, 0, wx.ALL | wx.EXPAND, 5)

        self.btn_coloreq = wx.Button(self.filterVpanel, wx.ID_ANY,
                                     _("Equalize"), size=(-1, -1))
        self.btn_coloreq.SetBitmap(bmpcoloreq, wx.LEFT)
        sizer_Vfilter.Add(self.btn_coloreq, 0, wx.ALL | wx.EXPAND, 5)

        self.box_Vfilters.Add(self.filterVpanel, 1, wx.EXPAND)
        self.filterVpanel.SetSizer(sizer_Vfilter)  # set panel
        self.filterVpanel.SetAutoLayout(1)
        self.filterVpanel.SetupScrolling()

        self.nb_Video.SetSizer(sizer_nbVideo)
        self.notebook.AddPage(self.nb_Video, _("Video"))
        #  END NOTEBOOK PANEL 1 Video

        # -------------- BEGIN NOTEBOOK PANEL 2 Audio:
        self.nb_Audio = wx.Panel(self.notebook, wx.ID_ANY)
        sizer_nbAudio = wx.BoxSizer(wx.HORIZONTAL)
        box5 = wx.StaticBox(self.nb_Audio, wx.ID_ANY, "")
        box_audio = wx.StaticBoxSizer(box5, wx.VERTICAL)
        sizer_nbAudio.Add(box_audio, 1, wx.ALL | wx.EXPAND, 5)
        self.audioenc = AudioEncoders(self.nb_Audio, self.opt, self.parent)
        box_audio.Add(self.audioenc, 1, wx.ALL | wx.EXPAND, 5)
        self.nb_Audio.SetSizer(sizer_nbAudio)
        self.notebook.AddPage(self.nb_Audio, _("Audio"))

        # -------------- BEGIN NOTEBOOK PANEL 3 Miscellaneous:
        self.nb_misc = wx.Panel(self.notebook, wx.ID_ANY)
        sizer_nbmisc = wx.BoxSizer(wx.HORIZONTAL)
        box6 = wx.StaticBox(self.nb_misc, wx.ID_ANY, "")
        box_misc = wx.StaticBoxSizer(box6, wx.VERTICAL)
        sizer_nbmisc.Add(box_misc, 1, wx.ALL | wx.EXPAND, 5)
        self.miscfunc = Miscellaneous(self.nb_misc, self.opt)
        box_misc.Add(self.miscfunc, 1, wx.ALL | wx.EXPAND, 5)
        self.nb_misc.SetSizer(sizer_nbmisc)
        self.notebook.AddPage(self.nb_misc, _("Miscellaneous"))

        # ------------------ set layout
        self.SetSizer(sizer_base)
        self.Fit()
        self.Layout()
        # ---------------------- Tooltips
        tip = _('Save the current settings as a new preset profile for the '
                'Preset Manager. You will find it in the Preset Manager '
                'where you can use it quickly.')
        self.btn_saveprst.SetToolTip(tip)
        tip = (_('Available video encoders. "Copy" means that the video '
                 'stream will not be re-encoded and will allow you (depending '
                 'on the encoder) to change the container, audio codec and a '
                 'few other parameters.'))
        self.cmb_vencoder.SetToolTip(tip)
        tip = (_('Container format. It is usually represented by the file '
                 'extension (output format).\n\nIf the Media target is set '
                 'to "Video" and the Video Encoder to "Copy", optionally you '
                 'can set this control to "Copy" the same container as '
                 'source file.\n\nIf the media target is set to "Audio", you '
                 'can extract only the audio streams from videos, or simply '
                 'convert audio source files.'))
        self.cmb_cont.SetToolTip(tip)
        tip = (_('Set output files target: "Video" to save output files as '
                 'video files; "Audio" to save files using an audio encoder '
                 'and format.\n\nNote that by using "Audio" as the target, '
                 'you will still be able to work on the video source files '
                 'but you will only be able to extract the audio streams, '
                 'convert them and apply audio filters such as '
                 'normalization.'))
        self.cmb_Media.SetToolTip(tip)
        tip = _('Preview video filters')
        self.btn_preview.SetToolTip(tip)
        tip = (_('Clear all enabled filters. Remove all video filter '
                 'data and apply default settings.'))
        self.btn_reset.SetToolTip(tip)

        # ----------------------Binding (EVT)----------------------#

        # Note: wx.EVT_TEXT_ENTER é diverso da wx.EVT_TEXT: Il primo
        # é responsivo agli input di tastiera, il secondo é responsivo
        # agli input di tastiera ma anche agli "append"

        self.Bind(wx.EVT_COMBOBOX, self.videoCodec, self.cmb_vencoder)
        self.Bind(wx.EVT_COMBOBOX, self.on_Container, self.cmb_cont)
        self.Bind(wx.EVT_COMBOBOX, self.on_Media, self.cmb_Media)
        self.Bind(wx.EVT_BUTTON, self.on_saveprst, self.btn_saveprst)
        self.Bind(wx.EVT_BUTTON, self.on_Set_scale, self.btn_videosize)
        self.Bind(wx.EVT_BUTTON, self.on_Set_crop, self.btn_crop)
        self.Bind(wx.EVT_BUTTON, self.on_Set_transpose, self.btn_rotate)
        self.Bind(wx.EVT_BUTTON, self.on_Set_deinterlace, self.btn_lacing)
        self.Bind(wx.EVT_BUTTON, self.on_Set_denoiser, self.btn_denois)
        self.Bind(wx.EVT_BUTTON, self.on_Set_stabilizer, self.btn_vidstab)
        self.Bind(wx.EVT_BUTTON, self.on_Set_coloreq, self.btn_coloreq)
        self.Bind(wx.EVT_BUTTON, self.on_video_preview, self.btn_preview)
        self.Bind(wx.EVT_BUTTON, self.on_vfilters_clear, self.btn_reset)
        self.Bind(wx.EVT_BUTTON, self.on_audio_preview,
                  self.audioenc.btn_audio_preview)

        #  initialize default layout:
        self.cmb_vencoder.SetSelection(2)
        self.cmb_Media.SetSelection(0), self.cmb_cont.SetSelection(0)
        self.vencoder_panel_set(default=True)
        self.audioenc.startup_one_time(), self.audioenc.normalize_default()
        pub.subscribe(self.reset_on_changed_data, "RESET_ON_CHANGED_LIST")
    # -------------------------------------------------------------------#

    def reset_on_changed_data(self, msg):
        """
        Called using pub/sub protocol.
        Be sure to call this method at the end of the `init` code
        above to allow the `AudioEncoders` class to set the
        required objects, e.g. some `opt` dictionary keys.
        """
        if self.opt["PEAK"] or self.opt["RMS"]:
            self.audioenc.normalize_default()
        if self.opt["VFilters"]:
            self.on_vfilters_clear(self)
    # -------------------------------------------------------------------#

    def vencoder_panel_set(self, default=False):
        """
        Show/Hide and Reset the correspondig panel of the specified
        encoder. Inizializing this class make sure to pass the
        `default=True` argument to this method.
        """
        if self.cmb_Media.GetValue() == 'Audio':
            self.videopanel.default(), self.videopanel.Hide()
            self.filterVpanel.Disable(), self.on_vfilters_clear(self)
            self.videopanel = self.disablevidpanels
            self.videopanel.Show(), self.videopanel.default()
            self.opt["CmdVideoParams"] = ''

        elif self.opt["VideoCodec"] == "-c:v libx264":  # default
            if not default:
                self.filterVpanel.Enable(), self.videopanel.Hide()
                self.videopanel = self.h264panel
                self.videopanel.Show()
            self.videopanel.default()

        elif self.opt["VideoCodec"] == "-c:v libx265":
            self.filterVpanel.Enable(), self.videopanel.Hide()
            self.videopanel = self.h265panel
            self.videopanel.Show(), self.videopanel.default()

        elif self.opt["VideoCodec"] == "-c:v libvpx-vp9":
            self.filterVpanel.Enable(), self.videopanel.Hide()
            self.videopanel = self.vp9panel
            self.videopanel.Show(), self.videopanel.on_reset_args(None)

        elif self.opt["VideoCodec"] == "-c:v libaom-av1":
            self.filterVpanel.Enable(), self.videopanel.Hide()
            self.videopanel = self.aompanel
            self.videopanel.Show(), self.videopanel.default()

        elif self.opt["VideoCodec"] == "-c:v libsvtav1":
            self.filterVpanel.Enable(), self.videopanel.Hide()
            self.videopanel = self.svtpanel
            self.videopanel.Show(), self.videopanel.on_reset_args(None)

        elif self.opt["VideoCodec"] in ("-c:v mpeg4", "-c:v libxvid"):
            self.filterVpanel.Enable(), self.videopanel.Hide()
            self.videopanel = self.mpeg4panel
            self.videopanel.Show(), self.videopanel.default()

        elif self.opt["VideoCodec"] == "-c:v copy":
            self.filterVpanel.Disable(), self.on_vfilters_clear(self)
            self.videopanel.Hide()
            self.videopanel = self.vcopypanel
            self.videopanel.Show(), self.videopanel.default()

        self.nb_Video.Layout()  # force Layout

    # ----------------------Event handler (callback)----------------------#

    def videoCodec(self, event):
        """
        This event triggers the setting to the default values.
        """
        selected = AV_Conv.VCODECS.get(self.cmb_vencoder.GetValue())
        libcodec = list(selected.keys())[0]
        self.cmb_cont.Clear()
        for f in selected.values():
            self.cmb_cont.Append((f),)
        self.cmb_cont.SetSelection(0)
        self.opt["VideoCodec"] = libcodec
        self.opt["VidCmbxStr"] = self.cmb_vencoder.GetValue()
        self.opt["OutputFormat"] = self.cmb_cont.GetValue()
        self.vencoder_panel_set()
        self.audioenc.audio_default()
        self.audioenc.set_audio_radiobox(None)
    # ------------------------------------------------------------------#

    def on_Media(self, event):
        """
        Combobox Media Sets layout to Audio or Video formats
        """
        if self.cmb_Media.GetValue() == 'Audio':
            self.opt["Media"] = 'Audio'
            self.cmb_vencoder.Disable()
            self.opt["VideoCodec"] = ''
            self.opt["VidCmbxStr"] = ''
            self.audioenc.audio_default()
            self.cmb_cont.Clear()
            for f in AV_Conv.A_FORMATS:
                self.cmb_cont.Append((f),)
            self.cmb_cont.SetSelection(0)
            self.opt["OutputFormat"] = self.cmb_cont.GetValue()
            self.vencoder_panel_set()
            self.audioenc.set_audio_radiobox(None)

        elif self.cmb_Media.GetValue() == 'Video':
            self.opt["Media"] = 'Video'
            self.cmb_vencoder.Enable()
            self.cmb_vencoder.SetSelection(2)
            self.videoCodec(self)
    # ------------------------------------------------------------------#

    def on_Container(self, event):
        """
        Appends on container combobox according to audio and video formats
        """
        if self.cmb_cont.GetValue() == "Copy":
            self.opt["OutputFormat"] = ''
        else:
            self.opt["OutputFormat"] = self.cmb_cont.GetValue()
        self.audioenc.set_audio_radiobox(None)
    # ------------------------------------------------------------------#

    def on_video_preview(self, event):
        """
        Showing selected video preview with applied filters.
        Note that libstab filter is not possible to preview.
        """
        fget = self.file_selection()
        if not fget or not self.opt["VFilters"]:
            return
        if self.opt["Vidstabtransform"]:
            wx.MessageBox(_("Unable to preview Video Stabilizer filter"),
                          "Videomass", wx.ICON_INFORMATION, self)
            return

        flt = self.opt["VFilters"]
        if self.parent.checktimestamp:
            flt = f'{flt},"{self.parent.cmdtimestamp}"'

        stream_play(self.parent.file_src[fget[1]],
                    self.parent.time_seq,
                    flt,
                    self.parent.autoexit
                    )
    # ------------------------------------------------------------------#

    def on_audio_preview(self, event):
        """
        Button event for button preview on `self.audioenc`.
        Only Bind it in this class.
        """
        fget = self.file_selection()
        self.audioenc.on_audio_preview(fget)
    # ------------------------------------------------------------------#

    def on_vfilters_clear(self, event):
        """
        Reset all enabled filters. If default disablevidstab
        arg is True, it disable only vidstab filter values.
        """
        if self.opt["VFilters"]:
            self.opt['Crop'], self.opt["Orientation"] = "", ["", ""]
            self.opt['Scale'], self.opt['Setdar'] = "", ""
            self.opt['Setsar'], self.opt['Deinterlace'] = "", ""
            self.opt['Interlace'], self.opt['Denoiser'] = "", ""
            self.opt["Vidstabtransform"], self.opt["Unsharp"] = "", ""
            self.opt["Vidstabdetect"], self.opt["Makeduo"] = "", False
            self.opt["VFilters"], self.opt["ColorEQ"] = "", ""

            self.btn_videosize.SetBackgroundColour(wx.NullColour)
            self.btn_crop.SetBackgroundColour(wx.NullColour)
            self.btn_denois.SetBackgroundColour(wx.NullColour)
            self.btn_lacing.SetBackgroundColour(wx.NullColour)
            self.btn_rotate.SetBackgroundColour(wx.NullColour)
            self.btn_vidstab.SetBackgroundColour(wx.NullColour)
            self.btn_coloreq.SetBackgroundColour(wx.NullColour)
            self.btn_preview.Disable()
            self.btn_reset.Disable()
    # ------------------------------------------------------------------#

    def file_selection(self):
        """
        Gets the selected file on files list and returns an object
        of type list [str('selected file name'), int(index)].
        Returns None if no files are selected.

        """
        if len(self.parent.file_src) == 1:
            return (self.parent.file_src[0], 0)

        if not self.parent.filedropselected:
            wx.MessageBox(_("Have to select an item in the file list first"),
                          'Videomass', wx.ICON_INFORMATION, self)
            return None

        clicked = self.parent.filedropselected
        return (clicked, self.parent.file_src.index(clicked))
    # ------------------------------------------------------------------#

    def get_video_stream(self):
        """
        Given a frame or a video file, it returns a dict object
        containing required video parameters as width, height, etc.
        """
        fget = self.file_selection()
        if not fget:
            return None

        index = self.parent.data_files[fget[1]]

        if 'video' in index.get('streams')[0]['codec_type']:
            width = int(index['streams'][0]['width'])
            height = int(index['streams'][0]['height'])
            filename = index['format']['filename']
            duration = index['format'].get('time', '00:00:00.000')
            if not width or not height:
                wx.MessageBox(_('Unsupported file:\n'
                                'Missing decoder or library? '
                                'Check FFmpeg configuration.'),
                              _('Videomass - Warning!'), wx.ICON_WARNING, self)
                self.on_vfilters_clear(self)
                return None
            return dict(zip(['width', 'height', 'filename', 'duration'],
                            [width, height, filename, duration]))

        wx.MessageBox(_('The file is not a frame or a video file'),
                      _('Videomass - Warning!'), wx.ICON_WARNING, self)
        self.on_vfilters_clear(self)
        return None
    # ------------------------------------------------------------------#

    def chain_all_video_filters(self):
        """
        Concatenate all video filters enabled and sorts
        them according to an consistance ffmpeg syntax.
        """
        orderf = (self.opt['Deinterlace'], self.opt['Interlace'],
                  self.opt["Denoiser"], self.opt["Vidstabtransform"],
                  self.opt["Unsharp"], self.opt['Crop'], self.opt['Scale'],
                  self.opt["Setdar"], self.opt["Setsar"],
                  self.opt['Orientation'][0], self.opt["ColorEQ"],
                  )  # do not change the order of the filters on this tuple
        filters = ''.join([f'{x},' for x in orderf if x])[:-1]

        if filters:
            self.opt["VFilters"] = f"-vf {filters}"
            self.btn_preview.Enable(), self.btn_reset.Enable()
        else:
            self.opt["VFilters"] = ""
            self.btn_preview.Disable(), self.btn_reset.Disable()
    # ------------------------------------------------------------------#

    def on_Set_scale(self, event):
        """
        Enable or disable scale, setdar and setsar filters
        """
        kwa = self.get_video_stream()
        if not kwa:
            return
        with Scale(self,
                   self.opt["Scale"],
                   self.opt["Setdar"],
                   self.opt["Setsar"],
                   self.bmpreset,
                   **kwa,
                   ) as sizing:
            if sizing.ShowModal() == wx.ID_OK:
                data = sizing.getvalue()
                if not [x for x in data.values() if x]:
                    self.btn_videosize.SetBackgroundColour(wx.NullColour)
                    self.opt["Setdar"] = ""
                    self.opt["Setsar"] = ""
                    self.opt["Scale"] = ""
                else:
                    self.btn_videosize.SetBackgroundColour(
                        wx.Colour(AV_Conv.VIOLET))
                    self.opt["Scale"] = data['scale']
                    self.opt['Setdar'] = data['setdar']
                    self.opt['Setsar'] = data['setsar']

                self.chain_all_video_filters()
    # -----------------------------------------------------------------#

    def on_Set_transpose(self, event):
        """
        Enable or disable transpose filter for frame rotations
        """
        kwa = self.get_video_stream()
        if not kwa:
            return
        with Transpose(self,
                       self.opt["Orientation"][0],
                       self.opt["Orientation"][1],
                       self.bmpreset,
                       **kwa,
                       ) as rotate:
            if rotate.ShowModal() == wx.ID_OK:
                data = rotate.getvalue()
                self.opt["Orientation"][0] = data[0]  # cmd option
                self.opt["Orientation"][1] = data[1]  # msg
                if not data[0]:
                    self.btn_rotate.SetBackgroundColour(wx.NullColour)
                else:
                    self.btn_rotate.SetBackgroundColour(
                        wx.Colour(AV_Conv.VIOLET))
                self.chain_all_video_filters()
    # ------------------------------------------------------------------#

    def on_Set_crop(self, event):
        """
        Enable or disable crop filter
        """
        kwa = self.get_video_stream()
        if not kwa:
            return
        with Crop(self, self.opt["Crop"], self.opt["CropColor"],
                  self.bmpreset, **kwa) as crop:
            if crop.ShowModal() == wx.ID_OK:
                data = crop.getvalue()
                if not data:
                    self.btn_crop.SetBackgroundColour(wx.NullColour)
                    self.opt["Crop"] = ''
                    self.opt["CropColor"] = ''
                else:
                    self.btn_crop.SetBackgroundColour(
                        wx.Colour(AV_Conv.VIOLET))
                    self.opt["Crop"] = f'crop={data[0]}'
                    self.opt["CropColor"] = data[1]
                self.chain_all_video_filters()
    # ------------------------------------------------------------------#

    def on_Set_deinterlace(self, event):
        """
        Enable or disable filter for deinterlacing
        (w3fdif and yadif) and interlace filter.
        """
        sdf = self.get_video_stream()
        if not sdf:
            return
        with Deinterlace(self,
                         self.opt["Deinterlace"],
                         self.opt["Interlace"],
                         self.bmpreset,
                         ) as lacing:
            if lacing.ShowModal() == wx.ID_OK:
                data = lacing.getvalue()
                if not data:
                    self.btn_lacing.SetBackgroundColour(wx.NullColour)
                    self.opt["Deinterlace"] = ''
                    self.opt["Interlace"] = ''
                else:
                    self.btn_lacing.SetBackgroundColour(
                        wx.Colour(AV_Conv.VIOLET))
                    if 'deinterlace' in data:
                        self.opt["Deinterlace"] = data["deinterlace"]
                        self.opt["Interlace"] = ''
                    elif 'interlace' in data:
                        self.opt["Interlace"] = data["interlace"]
                        self.opt["Deinterlace"] = ''
                self.chain_all_video_filters()
    # ------------------------------------------------------------------#

    def on_Set_denoiser(self, event):
        """
        Enable or disable denoiser filters (nlmeans and hqdn3d)
        useful in some case, i.e. when apply a deinterlace filter.
        <https://askubuntu.com/questions/866186/how-to-get-good-quality-when-
        converting-digital-video>
        """
        sdf = self.get_video_stream()
        if not sdf:
            return
        with Denoisers(self, self.opt["Denoiser"], self.bmpreset) as den:
            if den.ShowModal() == wx.ID_OK:
                data = den.getvalue()
                if not data:
                    self.btn_denois.SetBackgroundColour(wx.NullColour)
                    self.opt["Denoiser"] = ''
                else:
                    self.btn_denois.SetBackgroundColour(
                        wx.Colour(AV_Conv.VIOLET))
                    self.opt["Denoiser"] = data
                self.chain_all_video_filters()
    # ------------------------------------------------------------------#

    def on_Set_stabilizer(self, event):
        """
        Enable or disable libvidstab filter for video stabilization.
        Note, this filter is incompatible with two-pass encoding that
        includes `-pass 1` and` -pass 2` ffmpeg args/options.
        """
        sdf = self.get_video_stream()
        if not sdf:
            return
        with VidstabSet(self,
                        self.opt["Vidstabdetect"],
                        self.opt["Vidstabtransform"],
                        self.opt["Unsharp"],
                        self.opt["Makeduo"],
                        self.bmpreset,
                        **sdf,
                        ) as stab:
            if stab.ShowModal() == wx.ID_OK:
                data = stab.getvalue()
                if not data:
                    self.btn_vidstab.SetBackgroundColour(wx.NullColour)
                    self.opt["Vidstabdetect"] = ""
                    self.opt["Vidstabtransform"] = ""
                    self.opt["Unsharp"] = ""
                    self.opt["Makeduo"] = False
                else:
                    self.btn_vidstab.SetBackgroundColour(
                        wx.Colour(AV_Conv.VIOLET))
                    self.opt["Vidstabdetect"] = data[0]
                    self.opt['Vidstabtransform'] = data[1]
                    self.opt['Unsharp'] = data[2]
                    self.opt["Makeduo"] = data[3]
                self.chain_all_video_filters()
    # ------------------------------------------------------------------#

    def on_Set_coloreq(self, event):
        """
        Enable or disable color correction filter like
        contrast, brightness, saturation, gamma.
        """
        kwa = self.get_video_stream()
        if not kwa:
            return
        with ColorEQ(self, self.opt["ColorEQ"], self.bmpreset, **kwa,
                     ) as coloreq:
            if coloreq.ShowModal() == wx.ID_OK:
                data = coloreq.getvalue()
                if not data:
                    self.btn_coloreq.SetBackgroundColour(wx.NullColour)
                    self.opt["ColorEQ"] = ''
                else:
                    self.btn_coloreq.SetBackgroundColour(
                        wx.Colour(AV_Conv.VIOLET))
                    self.opt["ColorEQ"] = data
                self.chain_all_video_filters()
    # ------------------------------------------------------------------#

    def get_codec_args(self):
        """
        Get data from encoder panels and set ffmpeg
        command arguments
        """
        self.opt["CmdVideoParams"] = self.videopanel.video_options()
        self.opt["CmdAudioParams"] = self.audioenc.audio_options()

        if self.opt["Media"] == 'Video':

            if self.opt["Vidstabdetect"]:
                kwargs = self.video_stabilizer()

            elif self.opt["EBU"][0] == 'EBU R128 (High-Quality)':
                kwargs = self.video_ebu()
            else:
                kwargs = self.video_std()

        elif self.opt["Media"] == 'Audio':

            if self.opt["EBU"][0] == 'EBU R128 (High-Quality)':
                kwargs = self.audio_ebu()
            else:
                kwargs = self.audio_std()

        return kwargs
    # ------------------------------------------------------------------#

    def check_options(self, index=None):
        """
        Update entries and file check.
        """
        if self.audioenc.btn_voldect.IsEnabled():
            wx.MessageBox(_('Undetected volume values! Click the '
                            '"Volume detect" button to analyze '
                            'audio volume data.'),
                          'Videomass', wx.ICON_INFORMATION, self)
            return None

        if index is not None:
            infile = [self.parent.file_src[index]]
            outfilenames = [self.parent.outputnames[index]]
        else:
            infile = self.parent.file_src
            outfilenames = self.parent.outputnames

        filecheck = check_files(infile,
                                self.appdata['outputdir'],
                                self.appdata['outputdir_asinput'],
                                self.appdata['filesuffix'],
                                self.opt["OutputFormat"],
                                outfilenames,
                                )
        if not filecheck:  # User changing idea or not such files exist
            return None

        return filecheck
    # ------------------------------------------------------------------#

    def queue_mode(self):
        """
        build queue mode arguments. This method is
        called by `parent.on_add_to_queue`.
        Return a dictionary of data.
        """
        logname = 'Queue Processing.log'
        index = self.parent.file_src.index(self.parent.filedropselected)

        check = self.check_options(index)
        if not check:
            return None

        f_src, f_dest = check[0][0], check[1][0]
        kwargs = self.get_codec_args()
        dur, ss, et = update_timeseq_duration(self.parent.time_seq,
                                              self.parent.duration
                                              )
        kwargs['extension'] = self.opt["OutputFormat"]
        kwargs['pre-input-1'], kwargs['pre-input-2'] = '', ''
        kwargs['logname'] = logname
        kwargs['source'] = f_src
        kwargs['destination'] = f_dest
        kwargs["duration"] = dur[index]
        kwargs['start-time'], kwargs['end-time'] = ss, et
        if kwargs.get("volume"):
            kwargs["volume"] = kwargs["volume"][index]
        else:
            kwargs["volume"] = ''

        return kwargs
    # ------------------------------------------------------------------#

    def batch_mode(self):
        """
        build batch mode arguments. This method is called
        by `parent.click_start`
        """
        logname = 'AV Conversions.log'

        check = self.check_options()
        if not check:
            return None

        f_src, f_dest = check
        kwargs = self.get_codec_args()
        dur, ss, et = update_timeseq_duration(self.parent.time_seq,
                                              self.parent.duration
                                              )
        kwargs['extension'] = self.opt["OutputFormat"]
        kwargs['pre-input-1'], kwargs['pre-input-2'] = '', ''
        kwargs['logname'] = logname
        kwargs['start-time'], kwargs['end-time'] = ss, et
        batchlist = []
        for index in enumerate(self.parent.file_src):
            kw = kwargs.copy()
            kw['source'] = f_src[index[0]]
            kw['destination'] = f_dest[index[0]]
            kw['duration'] = dur[index[0]]
            if kw.get("volume"):
                kw["volume"] = kw["volume"][index[0]]
            else:
                kw["volume"] = ''
            batchlist.append(kw)

        keyval = self.update_dict(len(f_src), **kwargs)
        ending = Formula(self, (700, 250),
                         self.parent.movetotrash,
                         self.parent.emptylist,
                         **keyval,
                         )
        if ending.ShowModal() == wx.ID_OK:
            (self.parent.movetotrash,
             self.parent.emptylist) = ending.getvalue()
            self.parent.switch_to_processing(kwargs["type"],
                                             logname,
                                             datalist=batchlist
                                             )
        return None
    # ------------------------------------------------------------------#

    def video_stabilizer(self):
        """
        Build ffmpeg command strings for two pass
        video stabilizations process.

        """
        if self.opt["EBU"][0] == 'EBU R128 (High-Quality)':

            if self.opt["Passes"] == '2':
                cmd1 = (f'{self.opt["CmdVideoParams"]} '
                        f'-filter:v {self.opt["Vidstabdetect"]} '
                        f'{self.opt["passlogfile1"]} {self.opt["AudioIndex"]} '
                        f'-filter:a: {self.opt["EBU"][1]} -sn -dn '
                        f'-f {AV_Conv.MUXERS[self.opt["OutputFormat"]]}'
                        )
                cmd2 = (f'{self.opt["CmdVideoParams"]} {self.opt["VFilters"]} '
                        f'{self.opt["passlogfile2"]} '
                        f'{self.opt["CmdAudioParams"]} '
                        f'{self.opt["SubtitleMap"]} {self.opt["Chapters"]} '
                        f'{self.opt["MetaData"]}'
                        )
            else:  # single pass
                cmd1 = (f'-filter:v {self.opt["Vidstabdetect"]} '
                        f'{self.opt["AudioIndex"]} '
                        f'-filter:a: {self.opt["EBU"][1]} -sn -dn -f null'
                        )
                cmd2 = (f'{self.opt["CmdVideoParams"]} '
                        f'{self.opt["VFilters"]} {self.opt["CmdAudioParams"]} '
                        f'{self.opt["SubtitleMap"]} {self.opt["Chapters"]} '
                        f'{self.opt["MetaData"]}'
                        )
            pass1, pass2 = " ".join(cmd1.split()), " ".join(cmd2.split())
            kwargs = {'type': 'Two pass EBU', 'args': [pass1, pass2],
                      'EBU': self.opt["EBU"][1],
                      'audiomap': self.opt["AudioMap"],
                      'preset name':
                          'A/V Conversions - Video VIDSTAB/EBU Norm.',
                      }
        else:
            audnorm = (self.opt["RMS"] if not self.opt["PEAK"]
                       else self.opt["PEAK"])

            cmd1 = (f'-filter:v {self.opt["Vidstabdetect"]} '
                    f'-an -sn -dn -f null'
                    )
            cmd2 = (f'{self.opt["CmdVideoParams"]} {self.opt["VFilters"]} '
                    f'{self.opt["CmdAudioParams"]} {self.opt["EBU"][1]} '
                    f'{self.opt["SubtitleMap"]} {self.opt["Chapters"]} '
                    f'{self.opt["MetaData"]}'
                    )
            pass1, pass2 = " ".join(cmd1.split()), " ".join(cmd2.split())
            kwargs = {'type': 'Two pass VIDSTAB', 'args': [pass1, pass2],
                      'volume': [vol[5] for vol in audnorm],
                      'preset name': 'A/V Conversions - Video VIDSTAB.',
                      }
        return kwargs
        # ------------------------------------------------------------------#

    def video_std(self):
        """
        Build the ffmpeg args strings for standard
        video conversions.
        """
        audnorm = self.opt["RMS"] if not self.opt["PEAK"] else self.opt["PEAK"]

        if self.cmb_vencoder.GetValue() == "Copy":

            args = (f'{self.opt["CmdVideoParams"]} '
                    f'{self.opt["CmdAudioParams"]} {self.opt["EBU"][1]} '
                    f'{self.opt["SubtitleMap"]} {self.opt["Chapters"]} '
                    f'{self.opt["MetaData"]}'
                    )
            pass1, pass2 = " ".join(args.split()), ''
            kwargs = {'type': 'One pass', 'args': [pass1, pass2],
                      'volume': [vol[5] for vol in audnorm],
                      'preset name': 'A/V Conversions - Video standard',
                      }
        elif self.opt["Passes"] == "2":

            cmd1 = (f'{self.opt["CmdVideoParams"]} {self.opt["VFilters"]} '
                    f'{self.opt["passlogfile1"]} -an -sn -dn -f rawvideo'
                    )
            cmd2 = (f'{self.opt["CmdVideoParams"]} {self.opt["VFilters"]} '
                    f'{self.opt["passlogfile2"]} {self.opt["CmdAudioParams"]} '
                    f'{self.opt["EBU"][1]} {self.opt["SubtitleMap"]} '
                    f'{self.opt["Chapters"]} {self.opt["MetaData"]}'
                    )
            pass1, pass2 = " ".join(cmd1.split()), " ".join(cmd2.split())
            kwargs = {'type': 'Two pass', 'args': [pass1, pass2],
                      'volume': [vol[5] for vol in audnorm],
                      'preset name': 'A/V Conversions - Video standard.',
                      }
        elif self.opt["Passes"] == "Auto":
            args = (f'{self.opt["CmdVideoParams"]} {self.opt["VFilters"]} '
                    f'{self.opt["CmdAudioParams"]} {self.opt["EBU"][1]} '
                    f'{self.opt["SubtitleMap"]} {self.opt["Chapters"]} '
                    f'{self.opt["MetaData"]}'
                    )
            pass1, pass2 = " ".join(args.split()), ''
            kwargs = {'type': 'One pass', 'args': [pass1, pass2],
                      'volume': [vol[5] for vol in audnorm],
                      'preset name': 'A/V Conversions - Video standard',
                      }
        return kwargs
    # ------------------------------------------------------------------#

    def video_ebu(self):
        """
        Build the ffmpeg args strings for video conversions
        with two-pass EBU.
        NOTE If you want leave same indexes and process a selected Input Audio
             Index use same Output Audio Index on Audio Streams Mapping box

        """
        if self.cmb_vencoder.GetValue() == "Copy":
            cmd_1 = (f'{self.opt["AudioIndex"]} '
                     f'-filter:a: {self.opt["EBU"][1]} -vn -sn -dn  -f null'
                     )
            cmd_2 = (f'{self.opt["CmdVideoParams"]} {self.opt["VFilters"]} '
                     f'{self.opt["CmdAudioParams"]} {self.opt["SubtitleMap"]} '
                     f'{self.opt["Chapters"]} {self.opt["MetaData"]}'
                     )
            pass1 = " ".join(cmd_1.split())
            pass2 = " ".join(cmd_2.split())
            kwargs = {'type': 'Two pass EBU', 'args': [pass1, pass2],
                      'EBU': self.opt["EBU"][1],
                      'audiomap': self.opt["AudioMap"],
                      'preset name': 'A/V Conversions - copy Video/EBU Norm.',
                      }
        elif self.opt["Passes"] == "2":

            cmd_1 = (f'{self.opt["CmdVideoParams"]} {self.opt["VFilters"]} '
                     f'{self.opt["passlogfile1"]} {self.opt["AudioIndex"]} '
                     f'-filter:a: {self.opt["EBU"][1]} '
                     f'-sn -dn -f {AV_Conv.MUXERS[self.opt["OutputFormat"]]}'
                     )
            cmd_2 = (f'{self.opt["CmdVideoParams"]} {self.opt["VFilters"]} '
                     f'{self.opt["passlogfile2"]} '
                     f'{self.opt["CmdAudioParams"]} {self.opt["SubtitleMap"]} '
                     f'{self.opt["Chapters"]} {self.opt["MetaData"]}'
                     )
            pass1 = " ".join(cmd_1.split())
            pass2 = " ".join(cmd_2.split())
            kwargs = {'type': 'Two pass EBU', 'args': [pass1, pass2],
                      'EBU': self.opt["EBU"][1],
                      'audiomap': self.opt["AudioMap"],
                      'preset name': 'A/V Conversions - Video/EBU Norm.',
                      }
        else:
            cmd_1 = (f'{self.opt["AudioIndex"]} '
                     f'-filter:a: {self.opt["EBU"][1]} -vn -sn -dn -f null'
                     )
            cmd_2 = (f'{self.opt["CmdVideoParams"]} {self.opt["VFilters"]} '
                     f'{self.opt["CmdAudioParams"]} {self.opt["SubtitleMap"]} '
                     f'{self.opt["Chapters"]} {self.opt["MetaData"]}'
                     )
            pass1 = " ".join(cmd_1.split())
            pass2 = " ".join(cmd_2.split())
            kwargs = {'type': 'Two pass EBU', 'args': [pass1, pass2],
                      'EBU': self.opt["EBU"][1],
                      'audiomap': self.opt["AudioMap"],
                      'preset name': 'A/V Conversions - Video/EBU Norm.',
                      }
        return kwargs
    # ------------------------------------------------------------------#

    def audio_std(self):
        """
        Build the ffmpeg args strings for standard
        audio conversions.

        """
        audnorm = self.opt["RMS"] if not self.opt["PEAK"] else self.opt["PEAK"]

        args = (f'{self.opt["CmdAudioParams"]} '
                f'{self.opt["EBU"][1]} -vn -sn {self.opt["MetaData"]}'
                )
        pass1, pass2 = " ".join(args.split()), ''
        kwargs = {'type': 'One pass', 'args': [pass1, pass2],
                  'volume': [vol[5] for vol in audnorm],
                  'preset name': 'A/V Conversions - Audio standard',
                  }
        return kwargs
    # ------------------------------------------------------------------#

    def audio_ebu(self):
        """
        Build the ffmpeg args strings for audio conversions
        and EBU R128 normalization.
        WARNING do not map output audio file index on filter:a: , -c:a:
        and not send self.opt["AudioMap"] to process because the files
        audio has not indexes
        """
        cmd_1 = (f'{self.opt["AudioMap"][0]} -filter:a: {self.opt["EBU"][1]} '
                 f'-vn -sn -dn -f null'
                 )
        cmd_2 = (f'{self.opt["CmdAudioParams"]} -vn -sn {self.opt["MetaData"]}'
                 )
        pass1 = " ".join(cmd_1.split())
        pass2 = " ".join(cmd_2.split())
        kwargs = {'type': 'Two pass EBU', 'args': [pass1, pass2],
                  'EBU': self.opt["EBU"][1], 'audiomap': self.opt["AudioMap"],
                  'preset name': 'A/V Conversions - Audio/EBU Norm.',
                  }
        return kwargs
    # ------------------------------------------------------------------#

    def update_dict(self, countmax, **kwa):
        """
        Update to epilogue
        """
        if self.opt["PEAK"]:
            normalize = 'PEAK'
        elif self.opt["RMS"]:
            normalize = 'RMS'
        elif self.opt["EBU"]:
            normalize = self.opt["EBU"][0]
        else:
            normalize = _('Off')
        if self.cmb_cont.GetValue() == "Copy":
            outputformat = "Copy"
        else:
            outputformat = self.opt["OutputFormat"]
        if not self.parent.time_seq:
            sst, endt = _('Same as source'), _('Same as source')
        else:
            sst = kwa["start-time"].split()[1]
            endt = kwa["end-time"].split()[1]
        if self.appdata['outputdir_asinput']:
            dest = _('Same destination paths as source files')
        else:
            dest = self.appdata['outputdir']

        passes = '1' if kwa["args"][1] == '' else '2'

        keys = (_("Batch processing items\nDestination\nAutomation/Preset"
                  "\nEncoding passes\nOutput Format"
                  "\nVideo Codec\nAudio Codec\nAudio Normalization"
                  "\nOutput multimedia type\nStart of segment"
                  "\nClip duration"
                  ))
        vals = (f'{countmax}\n'
                f'{dest}\n'
                f'{kwa["preset name"]}\n'
                f'{passes}\n'
                f'{outputformat}\n'
                f'{self.opt["VidCmbxStr"]}\n'
                f'{self.opt["AudioCodStr"]}\n'
                f'{normalize}\n'
                f'{self.opt["Media"]}\n'
                f'{sst}\n'
                f'{endt}'
                )
        return {'key': keys, 'val': vals}
    # ------------------------------------------------------------------#

    def on_saveprst(self, event):
        """
        Save current setting as profile for the Presets
        Manager panel
        """
        if self.cmb_Media.GetValue() == 'Video':
            self.opt["CmdVideoParams"] = self.videopanel.video_options()
            self.opt["CmdAudioParams"] = self.audioenc.audio_options()

            if self.opt["Vidstabdetect"]:
                kwargs = self.video_stabilizer()
            elif self.opt["EBU"][0] == 'EBU R128 (High-Quality)':
                kwargs = self.video_ebu()
            else:
                kwargs = self.video_std()

        elif self.cmb_Media.GetValue() == 'Audio':
            self.opt["CmdAudioParams"] = self.audioenc.audio_options()
            if self.opt["EBU"][0] == 'EBU R128 (High-Quality)':
                kwargs = self.audio_ebu()
            else:
                kwargs = self.audio_std()

        with wx.FileDialog(
                None, _("Choose a Videomass preset..."),
                defaultDir=os.path.join(self.appdata['confdir'], 'presets'),
                wildcard="Videomass presets (*.json;)|*.json;",
                style=wx.FD_OPEN | wx.FD_FILE_MUST_EXIST) as fileDialog:

            if fileDialog.ShowModal() == wx.ID_CANCEL:
                return

            filename = os.path.splitext(fileDialog.GetPath())[0]
            basename = os.path.basename(filename)

            title = _('New Profile - Preset "{0}"').format(basename)

        args = kwargs["args"][0], kwargs["args"][1], self.opt["OutputFormat"]
        with setting_profiles.SettingProfile(self, 'addprofile',
                                             basename,
                                             args,
                                             title,
                                             ) as prstdialog:

            if prstdialog.ShowModal() == wx.ID_CANCEL:
                return
        self.parent.PrstsPanel.presets_refresh(self)



# -*- coding: UTF-8 -*-
"""
Name: ydl_extractinfo.py
Porpose: get informations data with youtube_dl
Compatibility: Python3, wxPython Phoenix
Author: Gianluca Pernigotto <jeanlucperni@gmail.com>
Copyleft - 2024 Gianluca Pernigotto <jeanlucperni@gmail.com>
license: GPL3
Rev: March.17.2023
Code checker: flake8, pylint

This file is part of Videomass.

   Videomass is free software: you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation, either version 3 of the License, or
   (at your option) any later version.

   Videomass is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with Videomass.  If not, see <http://www.gnu.org/licenses/>.
"""
from threading import Thread
import wx
from pubsub import pub
if wx.GetApp().appset['use-downloader']:
    import yt_dlp


class MyLogger:
    """
    Intercepts youtube-dl's output by setting a logger object .
    Log messages to a logging.Logger instance.
    <https://github.com/ytdl-org/youtube-dl/tree/3e4cedf9e8cd3157df2457df7274d0c842421945#embedding-youtube-dl>
    """
    def __init__(self):
        """
        make attribute to log messages error
        """
        self.msg_error = []

    def debug(self, msg):
        """
        Get debug messages
        """
        pass

    def warning(self, msg):
        """
        Get warning messages
        """
        pass

    def error(self, msg):
        """
        Get error messages
        """
        self.msg_error.append(msg)

    def get_message(self):
        """
        get message error from error method
        """
        return None if len(self.msg_error) == 0 else self.msg_error.pop()


class YdlExtractInfo(Thread):
    """
    Embed youtube-dl as module into a separated thread in order
    to get output during process (see help(youtube_dl.YoutubeDL) ) .

    """
    def __init__(self, url, kwargs):
        """
        Attributes defined here:
        self.url  str('url')
        self.data  tupla(None, None)
        """
        get = wx.GetApp()  # get videomass wx.App attribute
        self.appdata = get.appset
        self.url = url
        self.data = None
        self.kwargs = kwargs

        Thread.__init__(self)
        self.start()  # start the thread (va in self.run())

    def run(self):
        """
        Defines options to extract_info with youtube_dl
        """
        mylogger = MyLogger()
        ydl_opts = {**self.kwargs, 'logger': mylogger}
        with yt_dlp.YoutubeDL(ydl_opts) as ydl:
            meta = ydl.extract_info(self.url, download=False)
        error = mylogger.get_message()

        if error:
            self.data = (None, error)
        elif meta:
            self.data = (meta, None)

        wx.CallAfter(pub.sendMessage,
                     "RESULT_EVT",
                     status=''
                     )
